#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include "auxfun.c"
#include "linalg.c"

typedef struct data_penaltyfun 
{ 
  int *p;
  double *penalty;
} data_penaltyfun;

// lassoc performs Fu's lasso regression
// beta is p by 1 least squares estimate
// y is n by 1 outcome vector
// X is vectorized n by p design matrix 
// n is number of subjects
// p is number of predictors
// out argmin_{beta} (crossprod(y-X%*%beta))/2-n*sum(abs(beta))

void lassoc(double *beta, double *y, double *X, double *lambda, int *n, int *p, double *out)
{
  int j,k, iter=0;
  double eps = .00001;
  double Xmat[NMAX][NMAX], S0, diff, betaold[*p], tmp, betatmp[*p];
  double tX[NMAX][NMAX], tXy[NMAX], tXX[NMAX][NMAX], yvec[NMAX], maxiter=250;
  
  diff = eps+.01;
  matrix_unvecregvectobigmat(*n,*p,X,Xmat);
  vec_regvectobigvec(*n,y,yvec);
  matrix_transposetomat(*n,*p,Xmat,tX);
  matrix_multbyvec(*p,*n,tX,yvec,tXy);
  matrix_mult(*p,*n,*p,tX,Xmat,tXX);
  while ( diff > eps && iter < maxiter) {
    iter += 1;
    for (j=0;j<*p;j++)  {
      betaold[j] = beta[j];
    }
    for (j=0, diff = 0.0;j<*p;j++)  {
      for (k=0;k<*p;k++)  
	betatmp[k] = beta[k];
      betatmp[j] = 0.0;
      for (k=0,tmp=0;k<*p;k++) {
	tmp += tXX[j][k]*betatmp[k];
      }
      S0 = -2.0*(tXy[j] - tmp);
      if (S0 > *lambda)  
	beta[j] = (*lambda-S0)/(2.0*tXX[j][j]);
      else if (S0 < -*lambda) 
	beta[j] = (-*lambda-S0)/(2.0*tXX[j][j]);
      else
	beta[j] = 0.0;
      diff += pow(beta[j] - betaold[j],2.0); 
    }
    diff = sqrt(diff);
  }
  for (j=0;j<*p;j++)  
    out[j] = beta[j];
}

// scadpenalty_gradient calculates gradient of scad penalty function
// beta is p by 1 vector
// fundata should contain data_penaltyfun which should contain
//    p scalar
//    p by 1 vector of penalty parameters
// out = sum_{j=1}^p scad(beta[j],penalty[j])

void scadpenalty_gradient(double *beta, void *fundata, double *gradient)
{

  struct data_penaltyfun *mydata;
  mydata = fundata; 

  double a=3.7,*lambda;
  int j,p;

  lambda = mydata->penalty;
  p = *(mydata->p);
  
  for (j=0;j<p;j++) {
    if (lambda[j]==0)  {
      gradient[j] = 0.0;
      continue;
    }
    if (beta[j] < lambda[j])
      gradient[j] = lambda[j];
    else if (beta[j] < a*lambda[j])
      gradient[j] = (a*lambda[j]-beta[j])/(a-1);
    else
      gradient[j] = 0.0;
  }
}

// alassopenalty_gradient calculates gradient of function alasso penalty function
// beta is p by 1 vector
// fundata should contain data_penaltyfun which should contain
//    p scalar
//    p by 1 vector of penalty parameters
// out = sum_{j=1}^p alasso(beta[j],penalty[j])

void lassopenalty_gradient(double *beta, void *fundata, double *gradient)
{

  struct data_penaltyfun *mydata;
  mydata = fundata; 

  double *lambda;
  int j,p;

  lambda = mydata->penalty;
  p = *(mydata->p);
  for (j=0;j<p;j++) {
    gradient[j] = lambda[j];
  }

}

// nextbeta_mle computes next iterate of Newton Rhapson algorithm
// beta is p length vector previous iterate
// gradfun is function which takes argument fundata which computes gradient of function 
// hessfun is function which takes argument fundata which computes hessian of function 
// fundata is additinal arguments to funciton
// out = beta - solve(hessfun(beta,fundata))%*%gradfun(beta,fundata)

void nextbeta_mle(double *beta, void (*gradfun)(double *x, void *fundata, double *out), void (*hessfun)(double *x, void *fundata, double *out), 
		  void *fundata, int *p, double *out)
{
  int j;
  double hess[*p**p],hessmat[NMAX][NMAX],hessmatinv[NMAX][NMAX],grad[*p],deltabeta[*p];
  hessfun(beta,fundata,hess);
  matrix_unvecregvectobigmat(*p,*p,hess,hessmat);
  matrix_scale(*p,*p,-1.0,hessmat);
  ginv(*p,*p,hessmat,hessmatinv);
  matrix_scale(*p,*p,-1.0,hessmatinv);
  gradfun(beta,fundata,grad);
  rv_multbmbyrv(*p,*p,hessmatinv,grad,deltabeta);
  for (j=0;j<*p;j++)  
    out[j] = beta[j]-deltabeta[j];
}

// nextbeta_lqa computes next iterate of local quadratic approximation algorithm (Fan and Li, 2001)
// beta is p length vector of previous iterate
// gradfun is function which takes argument beta and fundata computes gradient of log likelihood function
// hessfun is function which takes argument beta and fundata computes gradient of log likelihood function
// fundata is additional arguments to functions gradfun and hessfun
// p is length of beta
// n is sample size
// penalize is p length vector indicator with a value of 1 if corresponding element in beta should be penalized
// penalty is penalty value
// penaltyfun is penaltyfunction
// out = equation (3.9) of Fan and Li (2001) JASSA

void nextbeta_lqa(double *beta, void (*gradfun)(double *x, void *fundata, double *out), void (*hessfun)(double *x, void *fundata, double *out), 
		  void *fundata, int *p, int *n, double *penalize, double *penalty, void (*penaltyfun)(double *x, void *fundata, double *out), 
		  double *out)
{
  int j,sumnot0,not0[*p],k,k1,k2,l;
  double hess[*p**p],hessmat[NMAX][NMAX],grad[NMAX],neghessmat[NMAX][NMAX],neghessmatinv[NMAX][NMAX];
  double eps[NMAX],absbeta[NMAX],scadgrad[NMAX];
  
  hessfun(beta,fundata,hess);
  matrix_unvecregvectobigmat(*p,*p,hess,hessmat);
  matrix_copy(*p,*p,hessmat,neghessmat);
  matrix_scale(*p,*p,-1.0,neghessmat);
  matrix_inversePD(*p,neghessmat,neghessmatinv);
  for (j=0;j<*p;j++)
    eps[j] = .1*sqrt(neghessmatinv[j][j]);
  for (j=0,sumnot0=0;j<*p;j++)  {
    not0[j] = fabs(beta[j]) > eps[j] || penalize[j]==0.0;
    sumnot0 += not0[j];
  }
  if (sumnot0==0)  {
    for (j=0;j<*p;j++)
      out[j] = 0;
    return;
  }
  for (j=0;j<*p;j++)
    absbeta[j] = fabs(beta[j]);
  
  struct data_penaltyfun data_pf;
  data_pf.penalty = penalty;
  data_pf.p = p;
  penaltyfun(absbeta,&data_pf,scadgrad);

  for (j=0;j<*p;j++)  
    scadgrad[j] = (penalize[j]==1) ? scadgrad[j] : 0;

  double Sigma0[NMAX][NMAX],betanot0[*p],hessmatnot0[NMAX][NMAX];
  matrix_setpartofbigmatto0(sumnot0,sumnot0,Sigma0);
  for (j=0,k=0;j<*p;j++)  {
    if (not0[j] == 1) {
      Sigma0[k][k] = scadgrad[j]/absbeta[j];
      betanot0[k] = beta[j];
      k++;
    }
  }
  for (j=0,k1=0;j<*p;j++)  {
    if (not0[j] == 1) {
      for (l=0,k2=0;l<*p;l++) {
	if (not0[l] == 1) {
	  hessmatnot0[k1][k2] = hessmat[j][l];
	  k2++;
	}
      }
      k1++;
    }
  }
  double mat1[NMAX][NMAX],vec1[NMAX],vec2[NMAX],mat2[NMAX][NMAX],vec3[NMAX];
  matrix_addscale(sumnot0,sumnot0,1.0,-(double) *n,hessmatnot0,Sigma0,mat1);
  gradfun(beta,fundata,grad);
  matrix_multbyvec(sumnot0,sumnot0,Sigma0,betanot0,vec1);
  for (j=0,k=0;j<*p;j++)  {
    if (not0[j] == 1) {
      vec2[k] = grad[j] - (*n)*vec1[k];
      k++;
    }
  }
  matrix_scale(sumnot0,sumnot0,-1.0,mat1);
  matrix_inversePD(sumnot0,mat1,mat2);  
  matrix_scale(sumnot0,sumnot0,-1.0,mat2);
  matrix_multbyvec(sumnot0,sumnot0,mat2,vec2,vec3);
  for (j=0,k=0;j<*p;j++)  {
    if ( not0[j] == 1) {
      out[j] = beta[j] - vec3[k];
      k++;
    } else {
      out[j] = 0;
    }
  }
}

// nextbeta_lla computes next iterate of local linear approximation algorithm (Zou and Li, 2008)
// beta is p length vector of previous iterate
// gradfun is function which takes argument beta and fundata computes gradient of log likelihood function
// hessfun is function which takes argument beta and fundata computes gradient of log likelihood function
// fundata is additional arguments to functions gradfun and hessfun
// p is length of beta
// n is sample size
// penalize is p length vector indicator with a value of 1 if corresponding element in beta should be penalized
// penalty is penalty value
// penaltyfun is penaltyfunction
// out = equation (3.9) of Zou and Li (2008) Annals

void nextbeta_lla(double *beta, void (*gradfun)(double *x, void *fundata, double *out), void (*hessfun)(double *x, void *fundata, double *out), 
		  void *fundata, int *p, int *n, double *penalize, double *penalty, void (*penaltyfun)(double *x, void *fundata, double *out), 
		  double *out)
{
  int gradbetaeq0[*p],sumgradbetaeq0,i,j;
  double absbeta[*p],pfgrad[*p],grad[*p],hess[(*p)*(*p)],hessmat[NMAX][NMAX];
  double Xstar[NMAX][NMAX], ystar2[NMAX], neghessmat[NMAX][NMAX],Xstar_inv[NMAX][NMAX];
  double ystar[NMAX],XstarM[NMAX][NMAX], XU[NMAX][NMAX], XVstar[NMAX][NMAX], HU[NMAX][NMAX];
  double tmpvec1[NMAX], tmpvec2[NMAX], tmpvec3[NMAX], tmpmat1[NMAX][NMAX];
  double ystarvec[*p],Xstarvec[(*p)*(*p)],M[NMAX][NMAX],XVstar2[NMAX][NMAX];
  double ystar2vec[*p],betahatU[NMAX],betahatV[NMAX],ystar3[NMAX],betals[NMAX];
  double tmpvec4[NMAX],lambdatmp[1];

  int tmp1,tmp2,inttmp[1];
  for (j=0;j<*p;j++)
    absbeta[j] = fabs(beta[j]);

  struct data_penaltyfun data_pf;
  data_pf.penalty = penalty;
  data_pf.p = p;
  penaltyfun(absbeta,&data_pf,pfgrad);

  for(j=0, sumgradbetaeq0 = 0;j<*p;j++)  {
    if (penalize[j] == 1 ) 
      gradbetaeq0[j] = (pfgrad[j]==0) ? 1 : 0;
    else 
      gradbetaeq0[j] = 1;
    sumgradbetaeq0 += gradbetaeq0[j];
  }
  hessfun(beta,fundata,hess);
  gradfun(beta,fundata,grad);  
  //vp(*p**p,hess,"hess");
  matrix_unvecregvectobigmat(*p,*p,hess,hessmat);
  
  matrix_copy(*p,*p,hessmat,neghessmat);
  matrix_scale(*p,*p,-1.0,neghessmat);
  //bmp(*p,*p,neghessmat,"neghessmat");
  matrix_cholesky(*p,neghessmat,Xstar);
  //bmp(*p,*p,Xstar,"Xstar");
  matrix_inverseLT(*p,Xstar,Xstar_inv);
  matrix_transpose(*p,*p,Xstar);
  matrix_multbyvec(*p,*p,Xstar,beta,tmpvec1);
  matrix_multbyvec(*p,*p,Xstar_inv,grad,tmpvec2);
  vec_addscale(*p,1.0,1.0,tmpvec1,tmpvec2,ystar);
  //vp(*p,ystar,"ystar");
  if (sumgradbetaeq0 == *p)  {
    matrix_lm(*p,*p,ystar,Xstar,tmpvec3);
    vec_bigvectoregvec(*p,tmpvec3,out);
    return;
  }
  matrix_setpartofbigmatto0(*p,*p,M);
  for (j=0;j<*p;j++) 
    M[j][j] = (gradbetaeq0[j] == 1) ? 1.0 : 1.0/pfgrad[j];
  matrix_mult(*p,*p,*p,Xstar,M,XstarM);
  //bmp(*p,*p,Xstar,"Xstar");
  //bmp(*p,*p,M,"M");
  //bmp(*p,*p,XstarM,"XstarM");
  if ( sumgradbetaeq0 == 0 )  {
    vec_bigvectoregvec(*p,ystar,ystarvec);
    matrix_vecbigmattoregvec(*p,*p,XstarM,Xstarvec);
    lambdatmp[0] = 2.0*(*n);
    matrix_lm(*p,*p,ystar,XstarM,betals);
    double betals2vec[*p];
    vec_bigvectoregvec(*p,betals,betals2vec);
    lassoc(betals2vec,ystarvec,Xstarvec,lambdatmp,p,p,out);
    for (j=0;j<*p;j++) 
      out[j] = out[j]*M[j][j];
    return;
  }
  for (i=0; i<*p; i++)  {
    for (j=0, tmp1=0, tmp2=0; j<*p; j++)  {
      if (gradbetaeq0[j] == 1 )  {
	XU[i][tmp1] = XstarM[i][j];
	tmp1++;
      } else {
	XVstar[i][tmp2] = XstarM[i][j];
	tmp2++;
      }
    }
  }
  //bmp(*p,sumgradbetaeq0,XU,"XU");
  matrix_proj_nd(*p,sumgradbetaeq0,XU,HU);
  //printf("p = %d, sumgradbetaeq0 = %d\n",p[0],sumgradbetaeq0);
  //bmp(*p,*p,HU,"HU");
  matrix_mult(*p,*p,*p-sumgradbetaeq0,HU,XVstar,tmpmat1);
  //bmp(*p,*p-sumgradbetaeq0,tmpmat1,"XVstar%*%HU");
  matrix_addscale(*p,*p-sumgradbetaeq0,1.0,-1.0,XVstar,tmpmat1,XVstar2);
  matrix_multbyvec(*p,*p,HU,ystar,tmpvec3);
  vec_addscale(*p,1.0,-1.0,ystar,tmpvec3,ystar2);
  vec_bigvectoregvec(*p,ystar2,ystar2vec);
  //vp(*p,ystar2vec,"ystar2vec");
  //for (j=0;j<*p;j++)
  //  printf("%f\n",ystar2[j]);
  //for (j=0;j<*p;j++)
  //  printf("%f\n",tmpystar2[j]);
  //vec_bigvectoregvec1(*p,ystar2,tmpystar2);
  //puts("transform");
  //for (j=0;j<*p;j++)
  //  printf("%f\n",ystar2[j]);
  //for (j=0;j<*p;j++)
  //  printf("%f\n",tmpystar2[j]);
  //bmp(*p,*p-sumgradbetaeq0,XVstar2,"XVstar2");
  double XVstar2vec[*p*(*p-sumgradbetaeq0)];
  matrix_vecbigmattoregvec(*p,*p-sumgradbetaeq0,XVstar2,XVstar2vec);
  //vp(*p,ystar2vec,"ystar2vec");
  //bmp(*p,*p-sumgradbetaeq0,XVstar2,"XVstar2");
  lambdatmp[0] = 2.0*(*n);
  inttmp[0] = *p - sumgradbetaeq0;
  //vp(*p,ystar2,"ystar2");
  matrix_lm(*p,*p-sumgradbetaeq0,ystar2,XVstar2,betals);
  //vp(*p,ystar2vec,"ystar2vec-afterlm");
  //vp(*p,ystar2,"ystar2");
  double betalsvec[*p-sumgradbetaeq0];
  //vp(*p,ystar2vec,"ystar2vec-afterlm2");
  //vp(*p-sumgradbetaeq0,betals,"betals");
  vec_bigvectoregvec1(*p-sumgradbetaeq0,betals,betalsvec);
  //vp(*p-sumgradbetaeq0,betalsvec,"betalsvec");
  //vp(*p,ystar2vec,"ystar2vec");
  //printf("p = %d, q = %d\n",p[0],inttmp[0]);
  //vp(*p-sumgradbetaeq0,betalsvec,"betals");
  //vp(*p,ystar2vec,"ystar2");
  //vectorprint(*p,ystar2vec,"ystar2");
  //vectorprint(*p,tmpystar2,"tmpystar2");
  //vp(*p,tmpystar2,"tmpystar2");
  //for (j=0;j<*p;j++)
  //  printf("%f\n",tmpystar2[j]);
  //vp(*p*inttmp[0],XVstar2vec,"XVstar2");
  //for (j=0;j<*p;j++)
  //  printf("%f\n",ystar2vec[j]);
  //vp(*p,ystar2vec,"ystar2vec");
  lassoc(betalsvec,ystar2vec,XVstar2vec,lambdatmp,p,inttmp,out);
  //vp(*p-sumgradbetaeq0,out,"betahatV");
  vec_regvectobigvec(*p-sumgradbetaeq0,out,betahatV);
  matrix_multbyvec(*p,*p-sumgradbetaeq0,XVstar,betahatV,tmpvec4);
  vec_addscale(*p,1.0,-1.0,ystar,tmpvec4,ystar3);
  matrix_lm(*p,sumgradbetaeq0,ystar3,XU,betahatU);
  for (j=0, tmp1=0, tmp2=0;j<*p;j++) {
    if ( gradbetaeq0[j] == 1 )  {
      out[j] = betahatU[tmp1];
      tmp1++;
    } else {
      out[j] = betahatV[tmp2]*M[j][j];
      tmp2++;
    }
  }
}

// groupdistance calculate the euclidean distance of a vector belonging to different groups
// theta p length vector
// p length of theta
// groupindex is p length vector which indicates which group corresponding element in theta belongs to
// numgroup is total number of groups
// NOTE: range of groupindex should be from 0 to numgroup-1
// Output:
// length is numgroup length vector, where ith element indicates total of thetas which belong to group i
// ditgance is numgroup length vector, where ith element is the euclidean distance of all of the elements belonging to group i
 


void groupdistance(double *theta, int *p, int *groupindex, int *numgroup, int *length, double *distance)  {

  int i,j,lengroup[*numgroup];
  double distgroup[*numgroup],ss;

  for (i=0;i<*numgroup;i++)  {
    for (j=0,lengroup[i]=0;j<*p;j++)  {
      if (groupindex[j]==i)
	lengroup[i]++;
    } 
    for (j=0,ss=0.0;j<*p;j++)  {
      if (groupindex[j]==i)
	ss += pow(theta[j],2.0);
      distgroup[i] = sqrt(ss);
    }
  }
  //ivp(*numgroup,lengroup,"lengroup");
  //vp(*numgroup,distgroup,"distgroup");

  for (j=0;j<*p;j++)  {
    length[j] = lengroup[groupindex[j]];
    distance[j] = distgroup[groupindex[j]];
  }
  //ivp(*p,length,"length");
  //vp(*p,distance,"distance");
}
  
// nextbeta_grouplqa computes next iterate of group local quadratic approximation algorithm of (Fan and Li, 2008)
// beta is p length vector of previous iterate
// gradfun is function which takes argument beta and fundata computes gradient of log likelihood function
// hessfun is function which takes argument beta and fundata computes gradient of log likelihood function
// fundata is additional arguments to functions gradfun and hessfun
// p is length of beta
// n is sample size
// penalize is p length vector indicator with a value of 1 if corresponding element in beta should be penalized
// penalty is penalty value
// groupindex is p length vector which indicates which group corresponding element in theta belongs to
// numgroup is total number of groups
// penaltyfun is penalty function
// out = group version of equation equation (3.9) of Fan and Li (2008) Jassa

void nextbeta_grouplqa(double *beta, void (*gradfun)(double *x, void *fundata, double *out), void (*hessfun)(double *x, void *fundata, double *out), 
		       void *fundata, int *p, int *n, double *penalize, double *penalty, int *groupindex,  int *numgroup,
		       void (*penaltyfun)(double *x, void *fundata, double *out), double *out)
{
  int j,sumnot0,not0[*p],k,k1,k2,l;
  double hess[*p**p],hessmat[NMAX][NMAX],grad[NMAX],neghessmat[NMAX][NMAX],neghessmatinv[NMAX][NMAX];
  double eps[NMAX],absbeta[NMAX],scadgrad[NMAX];

  int length[*p];
  double distance[*p],epsgroup[*p];
  
  //ivp(*p,groupindex,"groupdistance");
  //vp(*p,beta,"beta");
  groupdistance(beta,p,groupindex,numgroup,length,distance);
  //ivp(*p,length,"length");
  //vp(*p,distance,"distance");
  hessfun(beta,fundata,hess);
  matrix_unvecregvectobigmat(*p,*p,hess,hessmat);
  matrix_copy(*p,*p,hessmat,neghessmat);
  matrix_scale(*p,*p,-1.0,neghessmat);
  matrix_inversePD(*p,neghessmat,neghessmatinv);
  for (j=0;j<*p;j++)
    eps[j] = .1*sqrt(neghessmatinv[j][j]);
  groupdistance(eps,p,groupindex,numgroup,length,epsgroup);
  
  for (j=0,sumnot0=0;j<*p;j++)  {
    //not0[j] = fabs(distance[j]) > epsgroup[j] || penalize[j]==0.0;
    not0[j] = fabs(distance[j]) > pow(.025,.5) || penalize[j]==0.0;
    sumnot0 += not0[j];
  }
  if (sumnot0==0)  {
    for (j=0;j<*p;j++)
      out[j] = 0;
    return;
  }

  
  for (j=0;j<*p;j++)
    absbeta[j] = fabs(beta[j]);
  
  struct data_penaltyfun data_pf;
  data_pf.penalty = penalty;
  data_pf.p = p;


  double sqrtdist[*p]; 
  double lqaapprox[*p];
    
  for (j=0;j<*p;j++)  {
    //sqrtdist[j] = sqrt(length[j])*distance[j];
    sqrtdist[j] = sqrt(1)*distance[j];
  }
  penaltyfun(sqrtdist,&data_pf,scadgrad);
  //vp(*p,sqrtdist,"sqrtdist");
  //vp(*p,scadgrad,"scadgrad");
  //ivp(*p,length,"length");
  //vp(*p,distance,"distance");
  for (j=0;j<*p;j++)  {
    //lqaapprox[j] = sqrt(length[j])*scadgrad[j]/(distance[j]*length[j]);
    lqaapprox[j] = scadgrad[j]/(distance[j]*length[j]);
  }
  for (j=0;j<*p;j++)  
    lqaapprox[j] = (penalize[j]==1) ? lqaapprox[j] : 0.0;
  //vp(*p,lqaapprox,"lqaapprox");

  double Sigma0[NMAX][NMAX],betanot0[*p],hessmatnot0[NMAX][NMAX];
  matrix_setpartofbigmatto0(sumnot0,sumnot0,Sigma0);
  for (j=0,k=0;j<*p;j++)  {
    if (not0[j] == 1) {
      Sigma0[k][k] = lqaapprox[j];
      betanot0[k] = beta[j];
      k++;
    }
  }
  for (j=0,k1=0;j<*p;j++)  {
    if (not0[j] == 1) {
      for (l=0,k2=0;l<*p;l++) {
	if (not0[l] == 1) {
	  hessmatnot0[k1][k2] = hessmat[j][l];
	  k2++;
	}
      }
      k1++;
    }
  }
  double mat1[NMAX][NMAX],vec1[NMAX],vec2[NMAX],mat2[NMAX][NMAX],vec3[NMAX];
  matrix_addscale(sumnot0,sumnot0,1.0,-(double) *n,hessmatnot0,Sigma0,mat1);
  //bmp(sumnot0,sumnot0,mat1,"mat1");
  gradfun(beta,fundata,grad);
  matrix_multbyvec(sumnot0,sumnot0,Sigma0,betanot0,vec1);
  for (j=0,k=0;j<*p;j++)  {
    if (not0[j] == 1) {
      vec2[k] = grad[j] - (*n)*vec1[k];
      k++;
    }
  }
  matrix_scale(sumnot0,sumnot0,-1.0,mat1);
  matrix_inversePD(sumnot0,mat1,mat2);  
  matrix_scale(sumnot0,sumnot0,-1.0,mat2);
  matrix_multbyvec(sumnot0,sumnot0,mat2,vec2,vec3);
  //vp(sumnot0,vec3,"vec3");
  for (j=0,k=0;j<*p;j++)  {
    if ( not0[j] == 1) {
      out[j] = beta[j] - vec3[k];
      k++;
    } else {
      out[j] = 0;
    }
  }
}

// nextbeta_grouplla computes next iterate of group local linear approximation algorithm of (Zou and Li, 2008)
// beta is p length vector of previous iterate
// gradfun is function which takes argument beta and fundata computes gradient of log likelihood function
// hessfun is function which takes argument beta and fundata computes gradient of log likelihood function
// fundata is additional arguments to functions gradfun and hessfun
// p is length of beta
// n is sample size
// penalize is p length vector indicator with a value of 1 if corresponding element in beta should be penalized
// penalty is penalty value
// groupindex is p length vector which indicates which group corresponding element in theta belongs to
// numgroup is total number of groups
// penaltyfun is penalty function
// out = group version of equation (3.9) of Zou and Li (2001) Annals

void nextbeta_grouplla(double *beta, double *grad, double *hess, int *p, int *n, double *penalize, 
		       double *penalty, int *groupindex,  int *numgroup, 
		       void (*penaltyfun)(double *x, void *fundata, double *out), 
		       double *out)
{
  int gradbetaeq0[*p],sumgradbetaeq0,i,j;
  double absbeta[*p],pfgrad[*p],hessmat[NMAX][NMAX];
  double Xstar[NMAX][NMAX], ystar2[NMAX], neghessmat[NMAX][NMAX],Xstar_inv[NMAX][NMAX];
  double ystar[NMAX],XstarM[NMAX][NMAX], XU[NMAX][NMAX], XVstar[NMAX][NMAX], HU[NMAX][NMAX];
  double tmpvec1[NMAX], tmpvec2[NMAX], tmpvec3[NMAX], tmpmat1[NMAX][NMAX];
  double ystarvec[*p],Xstarvec[(*p)*(*p)],M[NMAX][NMAX],XVstar2[NMAX][NMAX];
  double ystar2vec[*p],betahatU[NMAX],betahatV[NMAX],ystar3[NMAX],betals[NMAX];
  double tmpvec4[NMAX],lambdatmp[1];
  double maxpenalty;
  maxpenalty=30;

  int length[*p];
  double distance[*p];
  
  groupdistance(beta,p,groupindex,numgroup,length,distance);
  //ivp(*p,groupindex,"groupindex");
  //vp(*p,beta,"beta");
  //ivp(*p,length,"length");
  //vp(*p,distance,"distance");


  if (sum(*p,distance)==0) {
    for (j=0;j<*p;j++) 
      out[j]=0;
  }
  // if there are groups wich are all zero then
  // lets restart with subset of problem
  
  //vp(*p**p,hess,"hess");
  matrix_unvecregvectobigmat(*p,*p,hess,hessmat);
  
  matrix_copy(*p,*p,hessmat,neghessmat);
  matrix_scale(*p,*p,-1.0,neghessmat);
  //bmp(*p,*p,neghessmat,"neghessmat");
  matrix_cholesky(*p,neghessmat,Xstar);
  //bmp(*p,*p,Xstar,"Xstar");
  matrix_inverseLT(*p,Xstar,Xstar_inv);
  matrix_transpose(*p,*p,Xstar);
  matrix_multbyvec(*p,*p,Xstar,beta,tmpvec1);
  matrix_multbyvec(*p,*p,Xstar_inv,grad,tmpvec2);
  vec_addscale(*p,1.0,1.0,tmpvec1,tmpvec2,ystar);
  //vp(*p,ystar,"ystar");
  int tmp1,tmp2,inttmp[1];
  for (j=0;j<*p;j++)
    absbeta[j] = fabs(beta[j]);

  double sqrtdist[*p]; 
  for (j=0;j<*p;j++)  {
    //sqrtdist[j] = sqrt(length[j])*distance[j];
    sqrtdist[j] = sqrt(1)*distance[j];
  }
  struct data_penaltyfun data_pf;
  data_pf.penalty = penalty;
  data_pf.p = p;
  penaltyfun(sqrtdist,&data_pf,pfgrad);
  double llaapprox[*p];
  for (j=0;j<*p;j++)  { 
    //llaapprox[j] = sqrt(length[j])*pfgrad[j]*absbeta[j]/distance[j];
    llaapprox[j] = sqrt(1)*pfgrad[j]*absbeta[j]/distance[j];
    if (distance[j]==0)
      llaapprox[j] = maxpenalty;
  }
  //vp(*p,llaapprox,"llaapprox");
  for(j=0, sumgradbetaeq0 = 0;j<*p;j++)  {
    if (penalize[j] == 1 ) 
      gradbetaeq0[j] = (llaapprox[j]==0) ? 1 : 0;
    else 
      gradbetaeq0[j] = 1;
    sumgradbetaeq0 += gradbetaeq0[j];
  }

  if (sumgradbetaeq0 == *p)  {
    matrix_lm(*p,*p,ystar,Xstar,tmpvec3);
    vec_bigvectoregvec(*p,tmpvec3,out);
    return;
  }
  matrix_setpartofbigmatto0(*p,*p,M);
  for (j=0;j<*p;j++) 
    M[j][j] = (gradbetaeq0[j] == 1) ? 1.0 : 1.0/llaapprox[j];
  matrix_mult(*p,*p,*p,Xstar,M,XstarM);
  //bmp(*p,*p,Xstar,"Xstar");
  //bmp(*p,*p,M,"M");
  //bmp(*p,*p,XstarM,"XstarM");

  if ( sumgradbetaeq0 == 0 )  {
    vec_bigvectoregvec(*p,ystar,ystarvec);
    matrix_vecbigmattoregvec(*p,*p,XstarM,Xstarvec);
    lambdatmp[0] = 2.0*(*n);
    matrix_lm(*p,*p,ystar,XstarM,betals);
    double betals2vec[*p];
    vec_bigvectoregvec(*p,betals,betals2vec);
    lassoc(betals2vec,ystarvec,Xstarvec,lambdatmp,p,p,out);
    for (j=0;j<*p;j++) 
      out[j] = out[j]*M[j][j];
    return;
  }
  for (i=0; i<*p; i++)  {
    for (j=0, tmp1=0, tmp2=0; j<*p; j++)  {
      if (gradbetaeq0[j] == 1 )  {
	XU[i][tmp1] = XstarM[i][j];
	tmp1++;
      } else {
	XVstar[i][tmp2] = XstarM[i][j];
	tmp2++;
      }
    }
  }
  //bmp(*p,sumgradbetaeq0,XU,"XU");
  matrix_proj_nd(*p,sumgradbetaeq0,XU,HU);
  //bmp(*p,*p,HU,"HU");
  matrix_mult(*p,*p,*p-sumgradbetaeq0,HU,XVstar,tmpmat1);
  //bmp(*p,*p-sumgradbetaeq0,tmpmat1,"XVstar%*%HU");
  matrix_addscale(*p,*p-sumgradbetaeq0,1.0,-1.0,XVstar,tmpmat1,XVstar2);
  matrix_multbyvec(*p,*p,HU,ystar,tmpvec3);
  vec_addscale(*p,1.0,-1.0,ystar,tmpvec3,ystar2);
  vec_bigvectoregvec(*p,ystar2,ystar2vec);
  //vp(*p,ystar2vec,"ystar2vec");
  //for (j=0;j<*p;j++)
  //  printf("%f\n",ystar2[j]);
  //for (j=0;j<*p;j++)
  //  printf("%f\n",tmpystar2[j]);
  //vec_bigvectoregvec1(*p,ystar2,tmpystar2);
  //puts("transform");
  //for (j=0;j<*p;j++)
  //  printf("%f\n",ystar2[j]);
  //for (j=0;j<*p;j++)
  //  printf("%f\n",tmpystar2[j]);
  //printf("p[0] = %d, *p=%d, sumgradbetaeq0 = %d\n",p[0],*p,sumgradbetaeq0);
  //bmp(*p,*p-sumgradbetaeq0,XVstar2,"XVstar2");
  //ps();
  //return;
  double XVstar2vec[*p*(*p-sumgradbetaeq0)];
  matrix_vecbigmattoregvec(*p,*p-sumgradbetaeq0,XVstar2,XVstar2vec);
  //vp(*p,ystar2vec,"ystar2vec");
  //bmp(*p,*p-sumgradbetaeq0,XVstar2,"XVstar2");
  //ps();
  //return;
  lambdatmp[0] = 2.0*(*n);
  //ps();
  //return;
  inttmp[0] = *p - sumgradbetaeq0;
  //vp(*p,ystar2,"ystar2");
  matrix_lm(*p,*p-sumgradbetaeq0,ystar2,XVstar2,betals);
  //vp(*p,ystar2vec,"ystar2vec-afterlm");
  //vp(*p,ystar2,"ystar2");
  double betalsvec[*p-sumgradbetaeq0];
  //vp(*p,ystar2vec,"ystar2vec-afterlm2");
  //vp(*p-sumgradbetaeq0,betals,"betals");
  vec_bigvectoregvec1(*p-sumgradbetaeq0,betals,betalsvec);
  //vp(*p-sumgradbetaeq0,betalsvec,"betalsvec");
  //vp(*p,ystar2vec,"ystar2vec");
  //printf("p = %d, q = %d\n",p[0],inttmp[0]);
  //vp(*p-sumgradbetaeq0,betalsvec,"betals");
  //vp(*p,ystar2vec,"ystar2");
  //vectorprint(*p,ystar2vec,"ystar2");
  //vectorprint(*p,tmpystar2,"tmpystar2");
  //vp(*p,tmpystar2,"tmpystar2");
  //for (j=0;j<*p;j++)
  //  printf("%f\n",tmpystar2[j]);
  //vp(*p*inttmp[0],XVstar2vec,"XVstar2");
  //for (j=0;j<*p;j++)
  //  printf("%f\n",ystar2vec[j]);
  //vp(*p,ystar2vec,"ystar2vec");
  lassoc(betalsvec,ystar2vec,XVstar2vec,lambdatmp,p,inttmp,out);
  //vp(*p-sumgradbetaeq0,out,"betahatV");
  vec_regvectobigvec(*p-sumgradbetaeq0,out,betahatV);
  matrix_multbyvec(*p,*p-sumgradbetaeq0,XVstar,betahatV,tmpvec4);
  vec_addscale(*p,1.0,-1.0,ystar,tmpvec4,ystar3);
  matrix_lm(*p,sumgradbetaeq0,ystar3,XU,betahatU);
  for (j=0, tmp1=0, tmp2=0;j<*p;j++) {
    if ( gradbetaeq0[j] == 1 )  {
      out[j] = betahatU[tmp1];
      tmp1++;
    } else {
      out[j] = betahatV[tmp2]*M[j][j];
      tmp2++;
    }
  }
}

// lpd_scad when computing random effects under the scad penalty function we assume the penalty componenet of penalized log-likelihood
// is a log density, lpd scad computes this portion of the penalty under possible group penalization

// beta is p length vector 
// p is length of beta
// nn is sample size
// penalize is p length vector indicator with a value of 1 if corresponding element in beta should be penalized
// penalty is penalty value
// groupindex is p length vector which indicates which group corresponding element in theta belongs to
// numgroup is total number of groups
// totalout = sum_{j=1}^{numgroup} penaltyfun(||beta[j]||_2,penalty[i])

void lpd_scad(double *beta, int *p, double *penalty, int *nn, int *groupindex, int *numgroup, int *penalize, double *totalout)  
{
  double infinity,a,betastar,n,out;
  n = (double) *nn;
  infinity = 1000;
  a = 3.7;
  betastar = 10;

  int length[*p],j;
  double distance[*p];
  
  groupdistance(beta,p,groupindex,numgroup,length,distance);
  double sqrtdist[*p]; 

  for (j=0;j<*p;j++)  {
    //sqrtdist[j] = sqrt(length[j])*distance[j];
    sqrtdist[j] = sqrt(1)*distance[j];
  }
  totalout[0] = 0.0;
  for (j=0;j<*p;j++)  {
    out = 0.0;
    if (penalty[j]==0) {
      out += -infinity;
    } else {
      if (sqrtdist[j] < penalty[j]) { 
	out += -n*fabs(sqrtdist[j])*penalty[j];
	//printf("11j=%d,out=%f\n",j,out);
      } else if (sqrtdist[j] < a*penalty[j])  {
	out += n*(pow(sqrtdist[j],2)-2.0*a*penalty[j]*fabs(sqrtdist[j])+pow(penalty[j],2))/(2.0*(a-1.0));
	//printf("22j=%d,out=%f\n",j,out);
      } else if (sqrtdist[j] < betastar)  {
	out += -n*(a+1.0)*pow(penalty[j],2)/2.0;
	//printf("j=%d,out=%f\n",j,out);
      }  else {
	out += -infinity;
	//printf("j=%d,out=%f\n",j,out);
      }
    }	  
    out = (penalize[j]==1) ? out : 0;
    //printf("j=%d,out=%f\n",j,out/length[j]);
    totalout[0] += out/length[j]; 
  }
}

// lpd_alasso when computing random effects under the alasso penalty function we assume the penalty componenet of penalized log-likelihood
// is a log density, lpd alasso computes this portion of the penalty under possible group penalization

// beta is p length vector 
// p is length of beta
// nn is sample size
// penalize is p length vector indicator with a value of 1 if corresponding element in beta should be penalized
// penalty is penalty value
// groupindex is p length vector which indicates which group corresponding element in theta belongs to
// numgroup is total number of groups
// totalout = sum_{j=1}^{numgroup} penaltyfun(||beta[j]||_2,penalty[i])

void lpd_lasso(double *beta, int *p, double *penalty, int *nn, int *groupindex, int *numgroup, int *penalize, double *totalout)  
{
  double infinity,a,out,betastar,n;
  n = (double) *nn;
  infinity = 1000;
  a = 3.7;
  betastar = 10;

  int length[*p],j;
  double distance[*p];
  
  groupdistance(beta,p,groupindex,numgroup,length,distance);
  double sqrtdist[*p]; 
  for (j=0;j<*p;j++)  {
    //sqrtdist[j] = sqrt(length[j])*distance[j];
    sqrtdist[j] = sqrt(1)*distance[j];
  }  
  totalout[0] = 0.0;

  for (j=0;j<*p;j++)  {
    out = 0.0;
    if (penalty[j]==0) {
      out += -infinity;
    } else {
      out += -n*penalty[j]*sqrtdist[j];
    }
    out = (penalize[j]==1) ? out : 0;
    totalout[0] += out/length[j]; 
  }
}
