
library(MASS)
library(nlme)

## Auxilliary functions used when implementing penalized maximization of GLMM

### lower triangular vec
### takes a lower triangular portion of matrix A
### input: q x q matrix X
### output: q(q+1)/2 vector vech(A)

ltvec <- function(A)  {
  q <- nrow(A)
  fun <- function(i) A[i:q,i]
  out <- lapply(1:q,fun)
  out <- unlist(out)
  out
}

### duplication matrix
### for any symetric m by m matrix A st Dvech(A)=vec(A)
### input: m
### output: D

dupmat <- function(m)  {
  fun <- function(i,j)  {
    tmp <- rep(0,m*(m+1)/2)
    tmp[(j-1)*m+i-j*(j-1)/2] <- 1
    mat <- matrix(0,m,m)
    mat[i,j] <- 1
    if (i==j)  out <- c(mat)%*%t(tmp) else out <- c(mat+t(mat))%*%t(tmp)
    out
  }
  out <- 0
  for (i in 1:m)  {
    for (j in 1:i)  {
      out <- out + fun(i,j)
    }
  }
  out
}

### duplication lower triangular matrix
### takes a lower triangular qxq matrix A st Dvech(A)=vec(A)
### input: q
### output: D

dupltmat <- function(m)  {
  out <- dupmat(m)
  q <- nrow(out)^(1/2)
  ind <- rep(FALSE,q^2)
  for ( i in 1:q)  {
    for ( j in 1:q)  {
      if ( i<j)  ind[i+(j-1)*q] <- TRUE
    }
  }
  out[ind,] <- 0
  out
}

## ltmat takes a vector of length q(q+1)/2 stacks it by row in the lower triangular portion of the q by q lower triangular matrix M 
## m is a q(q+1)/2 by 1 length vector
ltmat <- function(m)  {
  morig <- m
  q <- (-1+(1+8*length(m))^(1/2))/2
  out <- matrix(0,q,q)
  for (i in 1:q)  {
    out[i:q,i] <- m[1:(q-i+1)]
    ind <- rep(TRUE,length(m))
    ind[1:(q-i+1)] <- FALSE
    m <- m[ind]
  }
  out
}

## commmat computes commutation matrix of, i.e. vec(t(A)) = K_{mn}%*%vec(A)
## args
## m,n are integers
## output
commutmat <- function(m,n)  {
  tmp <- diag(0,m*n,m*n)
  for (i in 1:m)  {
    for (j in 1:n)  {
      Hij <- matrix(0,m,n)
      Hij[i,j] <- 1
      tmp <- tmp + kronecker(Hij,t(Hij))
    }
  }
  tmp
}

## mpgrad calculate the gradient of the Moore-Penrose inverse of m by n matrix X
## arg
## X, m by n matrix
## output
## the mn by mn matrix d vec(Xmp)/ d vec(X)^T
Xmp.grad <- function(X)  {
  m <- nrow(X)
  n <- ncol(X)
  Xmp <- ginv(X)
  out <- (kronecker(t(Xmp)%*%Xmp,diag(n)-Xmp%*%X)+kronecker(diag(m)-X%*%Xmp,Xmp%*%t(Xmp)))%*%commutmat(m,n)-kronecker(t(Xmp),Xmp)
  out <- out
  out
}

## XmptXmp.grad calculate the gradient of m by m matrix t(ginv(X))%*%ginv(X)
## arg
## X, m by n matrix
## output
## the m^2 by mn matrix d vec(t(ginv(X))%*%ginv(X))/ d vec(X)^T
XmptXmp.grad <- function(X)  {
  m <- nrow(X)
  n <- ncol(X)
  Xmp <- ginv(X)
  out <- (kronecker(t(Xmp),diag(m))%*%commutmat(n,m)+kronecker(diag(m),t(Xmp)))%*%Xmp.grad(X)
  out <- out
  out
}

## XtXmpt.grad calculate the gradient of n by n matrix t(X)%*%t(ginv(X))
## arg
## X, m by n matrix
## output
## the n^2 by mn matrix d vec(t(X)%*%t(ginv(X)))/ d vec(X)^T
XtXmpt.grad <- function(X)  {
  m <- nrow(X)
  n <- ncol(X)
  Xmp <- ginv(X)
  out <- kronecker((Xmp),diag(n))%*%commutmat(m,n)+kronecker(diag(n),t(X))%*%commutmat(n,m)%*%Xmp.grad(X)
  out <- out
  out
}

## XmptXt.grad calculate the gradient of m by m matrix t(ginv(X))%*%t(X)
## arg
## X, m by n matrix
## output
## the m^2 by mn matrix d vec(t(ginv(X))%*%t(X))/ d vec(X)^T
XmptXt.grad <- function(X)  {
  m <- nrow(X)
  n <- ncol(X)
  Xmp <- ginv(X)
  out <- kronecker((X),diag(m))%*%commutmat(n,m)%*%Xmp.grad(X)+kronecker(diag(m),t(Xmp))%*%commutmat(m,n)
  out <- out
  out
}

## XmpXmpt.grad calculate the gradient of n by n matrix ginv(X)%*%t(ginv(X))
## arg
## X, m by n matrix
## output
## the n^2 by mn matrix d vec(ginv(X)%*%t(ginv(X))) / d vec(X)^T
XmpXmpt.grad <- function(X)  {
  m <- nrow(X)
  n <- ncol(X)
  Xmp <- ginv(X)
  out <- (kronecker((Xmp),diag(n))+kronecker(diag(n),(Xmp))%*%commutmat(n,m))%*%Xmp.grad(X)
  out <- out
  out
}

## mphess calculates the hessian of the Moore-Penrose inverse of a m by n matrix X
## arg
## X, m by n matrix
## output
## a list where the ith element of the list is the l-th element  vec(Xmp)_{ij}/d vec(X) d vec(X)^T
Xmp.hess <- function(X)  {
  m <- nrow(X)
  n <- ncol(X)
  Xt <- t(X)
  Xmp <- ginv(X)
  Xmpt <- t(Xmp)
  tmp <- list()
  for (j in 1:n)  {
    indj <- rep(0,n)
    indj[j] <- 1
    for (i in 1:m)  {
      indi <- rep(0,m)
      indi[i] <- 1
      tmpy <- kronecker(t(kronecker(indj,indi)),diag(m*n))
      tmp[[i+(j-1)*m]] <- tmpy
    }
  }
  tmp1 <- kronecker(kronecker(diag(m),commutmat(n,m)),diag(n))
  tmp2 <- kronecker(kronecker(diag(m),commutmat(n,n)),diag(m))
  fun <- function(x) {
    mat1 <- kronecker(XmptXmp.grad(X),c(diag(n)-Xt%*%Xmpt))
    mat2 <- kronecker(c(Xmpt%*%Xmp),XtXmpt.grad(X))
    mat3 <- kronecker(XmptXt.grad(X),c(Xmp%*%Xmpt))
    mat4 <- kronecker(c(diag(m)-Xmpt%*%Xt),XmpXmpt.grad(X))
    mat5 <- kronecker(Xmp.grad(X),c(Xmpt))
    mat6 <- kronecker(c(Xmp),commutmat(n,m)%*%Xmp.grad(X))
    out <- commutmat(n,m)%*%x%*%tmp1%*%(mat1-mat2-mat3+mat4)-x%*%tmp2%*%(mat5+mat6)
    out
  }
  out <- lapply(tmp,fun)
  out
}

## mphess calculates the hessian of the Moore-Penrose inverse of a n by n matrix X
## arg
## X, n by n matrix
## output
## a list where the ith element of the list is the l-th element  vec(Xmp)_{ij}/d vec(X) d vec(X)^T
## I rewrote function so that it doesn not evalute the matrix kronecker(t(kronecker(indj,indi)),diag(m*n)) which can be huge
## I only wrote it for a 
Xmp.hess <- function(X)  {
  m <- nrow(X)
  n <- ncol(X)
  Xt <- t(X)
  Xmp <- ginv(X)
  Xmpt <- t(Xmp)
  tmp <- list()
  dXmptXmp <- XmptXmp.grad(X)
  dXtXmpt <- XtXmpt.grad(X)
  dXmptXt <- XmptXt.grad(X)
  dXmpXmpt <- XmpXmpt.grad(X)
  dXmp <- Xmp.grad(X)
  tmp <- list()
  for (j in 1:n)  {
    for (i in 1:m)  {
      tmp[[i+(j-1)*m]] <- list(i,j)
    }
  }
  fun <- function(y)  {
    tmpi <- y[[1]]
    tmpj <- y[[2]]
    num <- (m*(tmpj-1)+tmpi)
    i <- ifelse(num%%n==0,num%/%n,num%/%n+1)
    j <- num-(i-1)*n
    u <- rep(0,m)
    v <- rep(0,n)
    u[i] <- 1
    v[j] <- 1
    if (num!=n*(i-1)+j) {
      print("Hey")
      browser()
    }
    mat1 <- kronecker(t(u),diag(m))
    mat2 <- kronecker(t(v),diag(n))
    mat3 <- kronecker(t(u),diag(n))
    mat4 <- kronecker(diag(m),t(v))
    out11 <- kronecker((diag(n)-Xt%*%Xmpt)[,j],mat1%*%dXmptXmp)
    out22 <- kronecker(mat2%*%dXtXmpt,(Xmpt%*%Xmp)[,i])
    out33 <- kronecker((Xmp%*%Xmpt)[,j],mat1%*%dXmptXt)
    out44 <- kronecker(mat2%*%dXmpXmpt,(diag(m)-Xmpt%*%Xt)[,i])
    out55 <- kronecker(mat3%*%dXmp,Xmpt[,j])
    out66 <- kronecker(Xmp[,i],mat4%*%dXmp)
    out <- out11-out22-out33+out44-out55-out66
    out
  }
  out <- lapply(tmp,fun)
  out
}

Xmp.hess.tmp <- function(X)  {
  m <- nrow(X)
  n <- ncol(X)
  Xt <- t(X)
  Xmp <- ginv(X)
  Xmpt <- t(Xmp)
  tmp <- list()
  dXmptXmp <- XmptXmp.grad(X)
  dXtXmpt <- XtXmpt.grad(X)
  dXmptXt <- XmptXt.grad(X)
  dXmpXmpt <- XmpXmpt.grad(X)
  dXmp <- Xmp.grad(X)
  tmp <- list()
  for (j in 1:n)  {
    for (i in 1:m)  {
      tmp[[i+(j-1)*m]] <- list(i,j)
    }
  }
  fun <- function(y)  {
    tmpi <- y[[1]]
    tmpj <- y[[2]]
    num <- (m*(tmpj-1)+tmpi)
    i <- ifelse(num%%n==0,num%/%n,num%/%n+1)
    j <- num-(i-1)*n
    u <- rep(0,m)
    v <- rep(0,n)
    u[i] <- 1
    v[j] <- 1
    if (num!=n*(i-1)+j) {
      print("Hey")
      browser()
    }
    mat1 <- kronecker(t(u),diag(m))
    mat2 <- kronecker(t(v),diag(n))
    mat3 <- kronecker(t(u),diag(n))
    mat4 <- kronecker(diag(m),t(v))
    out11 <- kronecker((diag(n)-Xt%*%Xmpt)[,j],mat1%*%dXmptXmp)
    out22 <- kronecker(mat2%*%dXtXmpt,(Xmpt%*%Xmp)[,i])
    out33 <- kronecker((Xmp%*%Xmpt)[,j],mat1%*%dXmptXt)
    out44 <- kronecker(mat2%*%dXmpXmpt,(diag(m)-Xmpt%*%Xt)[,i])
    out55 <- kronecker(mat3%*%dXmp,Xmpt[,j])
    out66 <- kronecker(Xmp[,i],mat4%*%dXmp)
    out <- out11-out22-out33+out44-out55-out66
    browser()
    out
  }
  out <- lapply(tmp,fun)
  funier <- function(y)  {
    tmpi <- y[[1]]
    tmpj <- y[[2]]
    num <- (m*(tmpj-1)+tmpi)
    i <- ifelse(num%%n==0,num%/%n,num%/%n+1)
    j <- num-(i-1)*n
    out11 <- kronecker((diag(n)-Xt%*%Xmpt)[,j],(t(Xmp)%*%Xmp)[,i])
    out33 <- kronecker((Xmp%*%Xmpt)[,j],(diag(m)-Xmpt%*%Xt)[,i])
    out55 <- kronecker(Xmp[,i],Xmpt[,j])
    out <- out11+out33-out55
    out
  }
  outier <- sapply(tmp,funier)
  out
}

## Xinv.grad calculates the gradient of the inverse of a n by n matrix X
## arg
## X, n by n matrix
## output
## a list where the ith element of the list is the l-th element  vec(Xmp)_{ij}/d vec(X) d vec(X)^T
## I rewrote function so that it doesn not evalute the matrix kronecker(t(kronecker(indj,indi)),diag(m*n)) which can be huge
## I only wrote it for a 
Xinv.grad <- function(X)  {
  n <- ncol(X)
  Xinv <- solve(X)
  dXinv <- -kronecker(t(Xinv),Xinv)
  dXinv
}


## Xinv.hess calculates the hessian of the inverse of a n by n matrix X
## arg
## X, n by n matrix
## output
## a list where the ith element of the list is the l-th element  vec(Xmp)_{ij}/d vec(X) d vec(X)^T
## I rewrote function so that it doesn not evalute the matrix kronecker(t(kronecker(indj,indi)),diag(m*n)) which can be huge
## I only wrote it for a 
Xinv.hess <- function(X)  {
  n <- ncol(X)
  Xinv <- solve(X)
  dXinv <- -kronecker(t(Xinv),Xinv)
  tmp <- list()
  for (j in 1:n)  {
    for (i in 1:n)  {
      tmp[[i+(j-1)*n]] <- list(i,j)
    }
  }
  fun <- function(y)  {
    i <- y[[1]]
    j <- y[[2]]
    u <- rep(0,n)
    v <- rep(0,n)
    u[i] <- 1
    v[j] <- 1
    mat1 <- kronecker(t(u),diag(n))
    mat2 <- kronecker(t(v),diag(n))
    out11 <- kronecker(mat2%*%dXinv,(t(Xinv))[,i])
    out22 <- kronecker(Xinv[,j],mat1%*%commutmat(n,n)%*%dXinv)
    out <- -out11-out22
    out
  }
  out <- lapply(tmp,fun)
  out
}

## ltrowderive calculates derivative of lower triangular matrix M with respec to the lower row stacked portion
## arg
## m, lower triangular elements of M by row
## output
## d vec(M)/ t(d m), where m are the non-redundant row elements of M i.e. m = (m_{11}, m_{21},m_{22},m_{31},..m_{33},..m_{q1},m_{qq})
ltrow.grad <- function(q)  {
  M <- ltmat(1:(q*(q+1)/2))
  tmp <- c(M)
  funy <- function(i)  {
    ind <- rep(0,q*(q+1)/2)
    if (tmp[i]>0) ind[tmp[i]] <- 1
    ind
  }
  out <- t(sapply(1:length(tmp),funy))
  out
}

## outerprod calculates derivative of the outer product of a matrix M
## arg
## output
## d vec(M%*%t(M))/ t(d M), 
outerprod.grad <- function(M)  {
  q <- ncol(M)
  out1 <- kronecker(M,diag(q))
  out2 <- kronecker(diag(q),M)%*%commutmat(q,q)
  out <- out1+out2
  out <- out
  out
}

## outerprod calculates hessian of the outer product of a matrix M
## arg
## output
## d vec(M%*%t(M))/ t(d M), 
outerprod.hess <- function(q)  {
  tmplist <- list()
  for ( i in 1:q)  {
    for ( j in 1:q)  {
      mat <- matrix(0,q,q)
      mat[i,j] <- mat[i,j]+1
      mat[j,i] <- mat[j,i]+1
      tmplist[[i+(j-1)*q]] <- kronecker(diag(q),mat)    }

  }
  tmplist
}

## outerprodlt.grad calculates gradient of outper product of lower triangular matrix m d(vec(M%*%t(M)))/d(ltvec(M))
## d vec(M%*%t(M))/ t(d M), 
outerprodltmat.grad <- function(m)  {
  M <- ltmat(m)
  out <- outerprod.grad(M)%*%dupltmat(nrow(M))
  out
}

## outerprodlt.hess calculates hessian of outper product of lower triangular matrix m d(vec(M%*%t(M)))/d(ltvec(M))
## arg
## output
## d vec(M%*%t(M))/ t(d M),

outerprodltmat.hess <- function(q)  {
  tmplist <- outerprod.hess(q)
  Jq <- dupltmat(q)
  out <-lapply(tmplist,function(x) t(Jq)%*%x%*%Jq)
  out
}

dXmpdltmat.grad <- function(m)  {
  M <- ltmat(m)
  q <- nrow(M)
  D <- crossprod(t(M))
  dDmp <- Xmp.grad(D)
  dMMt <- outerprodltmat.grad(m)
  out <- dDmp%*%dMMt
  out
}

dXmpdltmat.hess <- function(m)  {
  M <- ltmat(m)
  q <- nrow(M)
  D <- crossprod(t(M))
  d2Dmp <- Xmp.hess(D)
  dMMt <- outerprodltmat.grad(m)
  mat1 <- lapply(d2Dmp,function(x) t(dMMt)%*%x%*%dMMt)
  dDmp <- Xmp.grad(D)
  d2MMt <- outerprodltmat.hess(q)
  funy <- function(x)  {
    fun <- function(i)  x[i]*d2MMt[[i]]
    out <- sapply(1:length(x),fun)
    out <- matrix(apply(out,1,sum),q*(q+1)/2,q*(q+1)/2)
    out
  }
  mat2 <- lapply(1:q^2,function(i) funy(dDmp[i,]))
  out <- lapply(1:q^2,function(i) mat1[[i]]+mat2[[i]])
  out
}

## listvec takes a vector of size nq and puts it in a list with elements each with the q elements
listvec <- function(x,n,q)  {
  x <- as.numeric(x)
  out <- lapply(1:n,function(i) x[(q*(i-1)+1):(q*i)])
  out <- lapply(out,function(x) matrix(x,ncol=1))
  out
}

## arrayvec takes a vector of size sum(n*r*s) and puts it in a list where the with i-th element is a r[i] by s[i] matrix
arrayvec <- function(x,n,r,s)  {
  x <- as.numeric(x)
  if (length(r)==1) r <- rep(r,n)
  if (length(s)==1) s <- rep(s,n)
  fun <- function(i)  {
    if (i!=1) seqq <- (sum(r[1:(i-1)]*s[1:(i-1)])+1):sum(r[1:(i)]*s[1:(i)]) else seqq <- 1:sum(r[1]*s[1])
    out <- matrix(x[seqq],r[i],s[i])
    out
  }
  out <- lapply(1:n,fun)
  out
}

## groupltvbyrow
groupltmatbyrow <- function(q)  {
  mat <- matrix(0,q,q)
  for (i in 1:q)  mat[i,] <- i
  out <- ltvec(mat)
  out
}

##
opltv <- function(m)  {
  out <- crossprod(t(ltmat(m)))
  out
}

postnd <- function(y,A1,C1,theta2,C2)  {
  D <- ginv(t(A1)%*%ginv(C1)%*%A1+ginv(C2))
  d <- t(A1)%*%ginv(C1)%*%y+ginv(C2)%*%theta2
  out <- list(mean=D%*%d,var=D)
  out
}

blockdiag <- function (m1, m2, p.tr = 0, p.ll = 0)
{
     ## p.tr and p.ll are padding values
     topleft <- m1
     topright <- matrix(p.tr, nrow(m1), ncol(m2))
     colnames(topright) <- colnames(m2)
     lowleft <- matrix(p.ll, nrow(m2), ncol(m1))
     lowright <- m2
     rbind(cbind(topleft, topright), cbind(lowleft, lowright))
}


## fun <- function(j) fdHess(1:6,function(x) c((crossprod(t(ltmat(x)))))[j],.relStep=1e-9)$gradient
## out1 <- t(sapply(1:9,fun))
## #debug(ltrowderiv)
## #debug(mpcholderiv)
## out2 <- cholderiv(1:6)

## fun <- function(j) fdHess(1:6,function(x) c(ginv(crossprod(t(ltmat(x)))))[j],.relStep=1e-9)$gradient
## out1 <- t(sapply(1:9,fun))
## #debug(ltrowderiv)
## #debug(mpcholderiv)
## out2 <- mpcholderiv(1:6)


## fun <- function(j) fdHess(c(X),function(x) c(ginv(matrix(x,m,n)))[j],.relStep=1e-7)$gra
## out1 <- t(sapply(1:(m*n),fun))
## out2 <- mpgrad(X)

## fun <- function(j) fdHess(c(X),function(x) c(crossprod(ginv(matrix(x,m,n))))[j],.relStep=1e-7)$gra
## out1 <- t(sapply(1:(m^2),fun))
## out2 <- XmptXmp.grad(X)

## funy <- function(X) t(X)%*%t(ginv(X))
## fun <- function(j) fdHess(c(X),function(x) c(funy(matrix(x,m,n)))[j],.relStep=1e-7)$gra
## out1 <- t(sapply(1:(n^2),fun))
## out2 <- XtXmpt.grad(X)

## funy <- function(X) t(ginv(X))%*%t((X))
## fun <- function(j) fdHess(c(X),function(x) c(funy(matrix(x,m,n)))[j],.relStep=1e-6)$gra
## out1 <- t(sapply(1:(m^2),fun))
## out2 <- XmptXt.grad(X)

## funy <- function(X) (ginv(X))%*%t(ginv(X))
## fun <- function(j) fdHess(c(X),function(x) c(funy(matrix(x,m,n)))[j],.relStep=1e-7)$gra
## out1 <- t(sapply(1:(n^2),fun))
## out2 <- XmpXmpt.grad(X)

## m <- 2
## n <- 3
## x <- rnorm(m*n,0,1)
## X <- matrix(x,m,n)
## fun <- function(j) fdHess(c(X),function(x) c(ginv(matrix(x,m,n)))[j],.relStep=1e-4)$Hess
## out1 <- t(sapply(1:(m*n),fun))
## out1 <- lapply(1:(m*n),function(x) matrix(out1[x,],m*n,m*n))
## #debug(Xmp.hess)
## out2 <- Xmp.hess(X)
## #debug(Xmp.hess2)
## out3 <- Xmp.hess2(X)
## out <- max(sapply(1:(m*n),function(i) max(abs(c(out2[[i]])-c(out3[[i]])))))
## browser()

##out <- lapply(1:12, 
#out1 <- mpderiv(matrix(1:9,4,3))

## determening whether Xmpgrad is ok
## fun <- function(X)  {
##   m <- nrow(X)
##   n <- ncol(X)
##   out <- .C("Xmpgradwrapper",as.integer(m),as.integer(n),as.double(c(X)),out=double(length=(n*m)^2))$out
##   out <- matrix(out,m*n,m*n)
##   out
## }

## m <- 2
## n <- 3
## x <- rnorm(m*n,0,1)
## X <- matrix(x,m,n)
## out <- fun(X)
## out <- Xmp.grad(X)-out
## browser()


## determening whether Xmpgrad is ok
## fun <- function(X)  {
##   m <- nrow(X)
##   n <- ncol(X)
##   out <- .C("XmptXmpgradwrapper",as.integer(m),as.integer(n),as.double(c(X)),out=double(length=m*m*m*n))$out
##   out <- matrix(out,m*m,m*n)
##   out
## }

## m <- 2
## n <- 3
## x <- rnorm(m*n,0,1)
## X <- matrix(x,m,n)
## out <- fun(X)
## out <- XmptXmp.grad(X)-out
## browser()

## determening whether Xmpgrad is ok
## fun <- function(X)  {
##   m <- nrow(X)
##   n <- ncol(X)
##   out <- .C("XtXmptgradwrapper",as.integer(m),as.integer(n),as.double(c(X)),out=double(length=n*n*m*n))$out
##   out <- matrix(out,n*n,m*n)
##   out
## }

## m <- 2
## n <- 3
## x <- rnorm(m*n,0,1)
## X <- matrix(x,m,n)
## out <- fun(X)
## out <- XtXmpt.grad(X)-out
## browser()

## determening whether Xmpgrad is ok
## fun <- function(X)  {
##   m <- nrow(X)
##   n <- ncol(X)
##   out <- .C("XmptXtgradwrapper",as.integer(m),as.integer(n),as.double(c(X)),out=double(length=m*m*m*n))$out
##   out <- matrix(out,m*m,m*n)
##   out
## }

## m <- 2
## n <- 3
## x <- rnorm(m*n,0,1)
## X <- matrix(x,m,n)
## out <- fun(X)
## out <- XmptXt.grad(X)-out
## browser()

## determening whether Xmpgrad is ok
## fun <- function(X)  {
##   m <- nrow(X)
##   n <- ncol(X)
##   out <- .C("XmpXmptgradwrapper",as.integer(m),as.integer(n),as.double(c(X)),out=double(length=n*n*m*n))$out
##   out <- matrix(out,n*n,m*n)
##   out
## }

## m <- 2
## n <- 3
## x <- rnorm(m*n,0,1)
## X <- matrix(x,m,n)
## out <- fun(X)
## out <- XmpXmpt.grad(X)-out
## browser()

## Determening whether Xmpgrad is ok
## fun <- function(m,n)  {
##   mn <- n*m
##   out <- .C("commutmatwrapper",as.integer(m),as.integer(n),out=double(length=(mn)^2))$out
##   out <- matrix(out,mn,mn)
##   out
## }

## m <- 4
## n <- 3
## x <- rnorm(m*n,0,1)
## X <- matrix(x,m,n)
## out <- fun(m,n)
## browser()
## #out <- XmpXmpt.grad(X)-out
## browser()


## Determening whether Xmpgrad is ok
## fun <- function(X)  {
##   m <- nrow(X)
##   n <- ncol(X)
##   mn <- n*m
##   out <- .C("Xmphesswrapper",as.integer(m),as.integer(n),as.double(c(X)),out=double(length=(mn)^3))$out
##   out <- lapply(1:(mn),function(i) matrix(out[((i-1)*(mn^2)+1):(i*(mn^2))],mn,mn))
##   out
## }

## m <- 2
## n <- 3
## x <- rnorm(m*n,0,1)
## X <- matrix(x,m,n)
## out1 <- fun(X)
## out2 <- Xmp.hess2(X)
## out <- sapply(1:(m*n),function(i) max(abs(c(out1[[i]])-c(out2[[i]]))))                                      
## browser()

## funy <- function(X)  {
##   m <- nrow(X)
##   n <- ncol(X)
##   mn <- n*m
##   out <- .C("Xmphesswrapper",as.integer(m),as.integer(n),as.double(c(X)),out=double(length=(mn)^3))$out
##   out <- lapply(1:(mn),function(i) matrix(out[((i-1)*(mn^2)+1):(i*(mn^2))],mn,mn))
##   out
## }
## m <- 2
## n <- 3
## x <- rnorm(m*n,0,1)
## X <- matrix(x,m,n)
## fun <- function(j) fdHess(c(X),function(x) c(ginv(matrix(x,m,n)))[j],.relStep=1e-3)$Hess
## out1 <- t(sapply(1:(m*n),fun))
## out1 <- lapply(1:(m*n),function(x) matrix(out1[x,],m*n,m*n))
## out2 <- funy(X)
## out <- max(sapply(1:(m*n),function(i) max(abs(c(out1[[i]])-c(out2[[i]])))))
## browser()



## n <- 4
## X <- matrix(rnorm(n^2),n,n)
## X <- X+t(X)
## out <- dupmat(n)%*%ltvec(X)-c(X)

## n <- 4
## X <- ltmat(rnorm(n*(n+1)/2)
## fun <- function(j) fdHess(c(X),function(x) c(crossprod(t(matrix(x,n,n))))[j],.relStep=1e-7)$gra
## out1 <- t(sapply(1:(n*n),fun))
## out2 <- outerprodderiv(X)
## out <- out1-out2

## n <- 4
## x <- rnorm(n*(n+1)/2)
## X <- ltmat(x)
## funy <- function(x)  {
##   X <- ltmat(x)
##   out <- ifelse(diag(X)!=0,1/diag(X),0)
##   out
## }
## fun <- function(X)  fdHess(c(X),funy,.relStep=1e-7)$gra
## dlogdiagM <- matrix(0,q,q)
## tmp <- ifelse(diag(M)!=0,1/diag(M),0)
## for (i in 1:q) dlogdiagM[i,i] <- tmp[i]  
## out1 <- fun(x)
## out2 <- ltvec(dlogdiagM)

## n <- 4
## X <- matrix(rnorm(n^2),n,n)
## X <- X+t(X)
## out <- dupmat(n)%*%ltvec(X)-c(X)

## n <- 4
## x <- rnorm(n*(n+1)/2)
## funy <- function(x) c(crossprod(t(ltmat(x))))
## fun <- function(j) fdHess(x,function(x) funy(x)[j],.relStep=1e-4)$He
## out1 <- lapply(1:n^2,fun)
## out2 <- outerprodltmat.hess(n)
## out <- sapply(1:n^2,function(i) sum(abs(out1[[i]]-out2[[i]])))
## browser()

## n <- 4
## x <- c(0,rnorm(n*(n+1)/2-1,3))+.01
## x <- c(crossprod(t(ltmat(x))))
## funy <- function(x) c(ginv(x))
## fun <- function(j) fdHess(x,function(x) funy(matrix(x,n,n))[j],.relStep=1e-5)$gr
## out1 <- t(sapply(1:n^2,fun))
## #debug(dXmpdltmat.hess)
## out3 <- .C("Xmpgradwrapper",as.integer(n),as.integer(n),as.double(x),out=double(length=n^4))$out
## out3 <- matrix(out3,n^2,n^2)
## out2 <- Xmp.grad(matrix(x,n,n))
## out12 <- sum(abs(out1-out2))
## out13 <- sum(abs(out1-out3))
## out23 <- sum(abs(out2-out3))
## browser()

## X <- matrix(x,n,n)
## d <- sqrt(diag(X))
## D <- diag(sqrt(diag(X)))
## Xs <- diag(1/d)%*%X%*%diag(1/d)
## out33 <- .C("Xmpgradwrapper",as.integer(n),as.integer(n),as.double(Xs),
##             out=double(length=n^4))$out
## out33 <- matrix(out33,n^2,n^2)
## out22 <-  Xmp.grad(Xs)
## dd <- kronecker(d,d)
## out33 <- diag(1/dd)%*%out33%*%diag(1/dd)
## out22 <- diag(1/dd)%*%out22%*%diag(1/dd)
## out122 <- sum(abs(out1-out22))
## out133 <- sum(abs(out1-out33))
## out233 <- sum(abs(out22-out33))
## browser()

## n <- 4
## x <- c(0,rnorm(n*(n+1)/2-1,3))+.1
## funy <- function(x) c(ginv(crossprod(t(ltmat(x)))))
## fun <- function(j) fdHess(x,function(x) funy(x)[j],.relStep=1e-5)$gr
## out1 <- t(sapply(1:n^2,fun))
## #debug(dXmpdltmat.hess)
## out3 <- .C("opmpgradltmatwrapper",as.integer(n),as.double(c(ltmat(x))),out=double(length=n^2*(n*(n+1)/2)))$out
## out3 <- matrix(out3,n^2,n*(n+1)/2)
## out2 <- dXmpdltmat.grad(x)
## out12 <- sum(abs(out1-out2))
## out13 <- sum(abs(out1-out3))
## out23 <- sum(abs(out2-out3))

## n <- 3
## x <- runif(n*(n+1)/2)
## x <- ltmat(x)
## x <- crossprod(t(x))
## diag(x) <- 1
## x <- c(x)
## funy <- function(x) c(solve(matrix(x,n,n)))
## fun <- function(j) fdHess(x,function(x) funy(x)[j],.relStep=1e-5)$He
## out1 <- lapply(1:n^2,fun)
## out2 <- Xinv.hess(matrix(x,n,n))
## ##out3 <- .C("Xmphesswrapper",as.integer(n),as.integer(n),as.double(x), out=double(length=n^4))$out
## ##out3 <- arrayvec(out3,n^2,n*(n+1)/2,n*(n+1)/2)
## out12 <- (sapply(1:n^2,function(i) max(abs((out1[[i]]-out2[[i]])))))
## ##out13 <- sum(sapply(1:n^2,function(i) sum(abs(out1[[i]]-out3[[i]]))))
## ##out23 <- sum(sapply(1:n^2,function(i) sum(abs(out2[[i]]-out3[[i]]))))
## browser()

## n <- 3
## x <- runif(n*(n+1)/2)
## x <- ltmat(x)
## x <- crossprod(t(x))
## diag(x) <- 1
## x[1,1] <- .001
## x <- c(x)
## funy <- function(x) c(ginv(matrix(x,n,n)))
## out4 <- .C("Xmphesswrapper",as.integer(n),as.integer(n),as.double(x),
##            out=double(length=n^6))$out
## out4 <- arrayvec(out4,n^2,n^2,n^2)
## out2 <- Xmp.hess(matrix(x,n,n))
## out3 <- Xinv.hess(matrix(x,n,n))
## out12 <- out13 <- out23 <- out14 <- out24 <- out34 <- vector()
## for ( j in 1:8)  {
##   step <- c(1e-1,1e-2,1e-3,1e-4,1e-5,1e-6,1e-7,1e-8)[j]
##   fun <- function(j) fdHess(x,function(x) funy(x)[j],.relStep=step)$He
##   out1 <- lapply(1:n^2,fun)
##   out3 <- arrayvec(out3,n^2,n*(n+1)/2,n*(n+1)/2)
##   out12[j] <- sum(sapply(1:n^2,function(i) max(abs(out1[[i]]-out2[[i]]))))
##   out13[j] <- sum(sapply(1:n^2,function(i) max(abs(out1[[i]]-out3[[i]]))))
##   out14[j] <- sum(sapply(1:n^2,function(i) max(abs(out1[[i]]-out4[[i]]))))
##   out23[j] <- sum(sapply(1:n^2,function(i) max(abs(out2[[i]]-out3[[i]]))))
##   out24[j] <- sum(sapply(1:n^2,function(i) max(abs(out2[[i]]-out4[[i]]))))
##   out34[j] <- sum(sapply(1:n^2,function(i) max(abs(out3[[i]]-out4[[i]]))))
## }
## Xmp.hess.tmp(matrix(x,n,n))
## browser()

## n <- 3
## x <- rnorm(n^2)
## x <- c(x)
## funy <- function(x) c(ginv(matrix(x,n,n)))
## out4 <- .C("Xmphesswrapper",as.integer(n),as.integer(n),as.double(x),
##            out=double(length=n^6))$out
## out4 <- arrayvec(out4,n^2,n^2,n^2)
## out2 <- Xmp.hess(matrix(x,n,n))
## out3 <- Xinv.hess(matrix(x,n,n))
## out12 <- out13 <- out23 <- out14 <- out24 <- out34 <- vector()
## for ( j in 1:8)  {
##   step <- c(1e-1,1e-2,1e-3,1e-4,1e-5,1e-6,1e-7,1e-8)[j]
##   fun <- function(j) fdHess(x,function(x) funy(x)[j],.relStep=step)$He
##   out1 <- lapply(1:n^2,fun)
##   ##out3 <- arrayvec(out3,n^2,n*(n+1)/2,n*(n+1)/2)
##   out12[j] <- sum(sapply(1:n^2,function(i) max(abs(out1[[i]]-out2[[i]]))))
##   out13[j] <- sum(sapply(1:n^2,function(i) max(abs(out1[[i]]-out3[[i]]))))
##   out14[j] <- sum(sapply(1:n^2,function(i) max(abs(out1[[i]]-out4[[i]]))))
##   out23[j] <- sum(sapply(1:n^2,function(i) max(abs(out2[[i]]-out3[[i]]))))
##   out24[j] <- sum(sapply(1:n^2,function(i) max(abs(out2[[i]]-out4[[i]]))))
##   out34[j] <- sum(sapply(1:n^2,function(i) max(abs(out3[[i]]-out4[[i]]))))
## }
## Xmp.hess.tmp(matrix(x,n,n))

## n <- 3
## x <- c(.1,rnorm(n*(n+1)/2-1,))
## funy <- function(x) c(ginv(crossprod(t(ltmat(x)))))
## fun <- function(j) fdHess(x,function(x) funy(x)[j],.relStep=1e-4)$He
## out2 <- dXmpdltmat.hess(x)
## out3 <- .C("opmphessltmatwrapper",as.integer(n),as.double(c(ltmat(x))), out=double(length=n^2*(n*(n+1)/2)^2))$out
## out3 <- arrayvec(out3,n^2,n*(n+1)/2,n*(n+1)/2)
## out12 <- out13 <- out23 <- vector()
## for ( j in 1:8)  {
##   step <- c(1e-1,1e-2,1e-3,1e-4,1e-5,1e-6,1e-7,1e-8)[j]
##   fun <- function(j) fdHess(x,function(x) funy(x)[j],.relStep=step)$He
##   out1 <- lapply(1:n^2,fun)
##   out12[j] <- sum(sapply(1:n^2,function(i) max(abs(out1[[i]]-out2[[i]]))))
##   out13[j] <- sum(sapply(1:n^2,function(i) max(abs(out1[[i]]-out3[[i]]))))
##   out23[j] <- sum(sapply(1:n^2,function(i) max(abs(out2[[i]]-out3[[i]]))))
## }
## browser()


## fun <- function(q)  {
##   out <- .C("dupltmatwrapper",as.integer(q),out=double(length=q^2*q*(q+1)/2))$out
##   out <- matrix(out,q^2,q*(q+1)/2)
##   out
## }

## n <- 6
## out <- dupltmat(n) - fun(n)

## n <- 5
## m <- rnorm(n*(n+1)/2)
## fun <- function(m)  {
##   M <- ltmat(m)
##   n <- nrow(M)
##   out <- .C("outercrossprodltmatgradwrapper",as.integer(n),as.double(c(M)),out=double(length=n^2*n*(n+1)/2))$out
##   out <- matrix(out,n^2,n*(n+1)/2)
##   out
## }
## out <- outerprodltmat.grad(m) - fun(m)


##m <- c(-0.2319854,2.3001479,0.1131286)
##Xmphess(ltmat(m))
## m <- c(-0.2319854,2.3001479,0.1131286)
## D <- crossprod(t(ltmat(m)))
## n <- nrow(D)
## out1 <- .C("Xmphesswrapper",as.integer(2),as.integer(2),as.double(c(D)),out=double(n^2*(n^2)*n^2))$out
## out1 <- arrayvec(out1,n^2,n^2,n^2)
## out2 <- Xmp.hess(D) 
## out <- sapply(n^2,function(i) sum(abs(out1[[i]]-out2[[i]])))
## browser()
## m <- c(1,0,0)

## funy <- function(m)  {
##   out <- c(ginv(crossprod(t(ltmat(m)))))
##   out
## }
  
## fun <- function(j) fdHess(m,function(x) funy(x)[j],.relStep=1e-5)$He
## funs <- function(x)  {
##   M <- ltmat(x)
##   n <- nrow(M)
##   out <- .C("opmphessltmatwrapper",as.integer(n),as.double(c(M)), out=double(length=n^2*(n*(n+1)/2)^2))$out
##   out <- arrayvec(out,n^2,n*(n+1)/2,n*(n+1)/2)
##   out
##   out
## }

## n <- 3
## m <- rnorm(n*(n+1)/2,0,1)
## out2 <- funs(m)
## out1 <- dXmpdltmat.hess(m)
## out <- sapply(n^2,function(i) sum(abs(out1[[i]]-out2[[i]])))
## browser()
##dyn.unload("lasso.dll")

## M <- matrix(c(0.7071068,0,0,0,
##               0.2121320,0.4527693,0,0,
##               0,0,.01,0,
##               0,0,0,.01),4,4,byrow=TRUE)
## x <- ltvec(M)
## n <- nrow(M)
## x <- x+.1
## y <- crossprod(t(ltmat(x)))
## funy <- function(x) c(ginv((x)))
## out2 <- Xmp.grad(y)
## out3 <- .C("Xmpgradwrapper",as.integer(n),as.integer(n),as.double(y),
##            out=double(length=n^2*n^2))$out
## out3 <- matrix(out3,n^2,n^2)
## out12 <- out13 <- out23 <- vector()
## for ( j in 1:8)  {
##   step <- c(1e-1,1e-2,1e-3,1e-4,1e-5,1e-6,1e-7,1e-8)[j]
##   out1 <- jacobian(funy,y)
##   out12[j] <- (sapply(1:n^2,function(i) max(abs(out1-out2))))
##   out13[j] <- (sapply(1:n^2,function(i) max(abs(out1-out3))))
## }
## out23 <- sapply(1:n^2,function(i) max(abs(out2-out3)))
## browser()


## M <- matrix(c(0.7071068,0,0,0,
##               0.2121320,0.4527693,0,0,
##               0,0,.1,0,
##               0,0,0,.1),4,4,byrow=TRUE)
## x <- ltvec(M)
## n <- nrow(M)
## y <- crossprod(t(ltmat(x)))
## funy <- function(x) c(ginv((x)))
## out2 <- Xmp.hess(y)
## out3 <- .C("Xmphesswrapper",as.integer(n),as.integer(n),as.double(y),
##            out=double(length=n^2*n^2*n^2))$out
## out3 <- arrayvec(out3,n^2,n^2,n^2)
## fun <- function(j) jacobian(function(y) Xmp.grad(y)[j,],y)
## out1 <- lapply(1:n^2,fun)
## out12 <- sum(sapply(1:n^2,function(i) max(abs(out1[[i]]-out2[[i]]))))
## out13 <- sum(sapply(1:n^2,function(i) max(abs(out1[[i]]-out3[[i]]))))
## out23 <- sapply(1:n^2,function(i) max(abs(out2[[i]]-out3[[i]])))
## browser()

## M <- matrix(c(0.7071068,0,0,0,
##               0.2121320,0.4527693,0,0,
##               0,0,.1,0,
##               0,0,0,.1),4,4,byrow=TRUE)
## x <- ltvec(M)
## n <- nrow(M)
## out2 <- dXmpdltmat.grad(x)
## out3 <- .C("opmpgradltmatwrapper",as.integer(n),as.double(c(M)),
##            out=double(length=n^2*(n*(n+1)/2)))$out
## #out3 <- arrayvec(out3,n^2,n*(n+1)/2,n*(n+1)/2)
## fun <- function(y) c(ginv(crossprod(t(ltmat(y)))))
## out1 <- jacobian(fun,x)
## out12 <- sum(sapply(1:n^2,function(i) max(abs(out1[[i]]-out2[[i]]))))
## out13 <- sum(sapply(1:n^2,function(i) max(abs(out1[[i]]-out3[[i]]))))
## out23 <- sapply(1:n^2,function(i) max(abs(out2[[i]]-out3[[i]])))
## browser()


## M <- matrix(c(0.7071068,0,0,0,
##               0.2121320,0.4527693,0,0,
##               0,0,.1,0,
##               0,0,0,.1),4,4,byrow=TRUE)
## x <- ltvec(M)
## n <- nrow(M)
## out2 <- dXmpdltmat.hess(x)
## out3 <- .C("opmphessltmatwrapper",as.integer(n),as.double(c(M)),
##            out=double(length=n^2*(n*(n+1)/2)^2))$out
## out3 <- arrayvec(out3,n^2,n*(n+1)/2,n*(n+1)/2)
## fun <- function(j) jacobian(function(x) dXmpdltmat.grad(x)[j,],x)
## out1 <- lapply(1:n^2,fun)
## out12 <- sum(sapply(1:n^2,function(i) max(abs(out1[[i]]-out2[[i]]))))
## out13 <- sum(sapply(1:n^2,function(i) max(abs(out1[[i]]-out3[[i]]))))
## out23 <- sapply(1:n^2,function(i) max(abs(out2[[i]]-out3[[i]])))
## browser()

