/* lmeq.c   02/21/09
** programs which computes Q function of generalized linear mixed effects model 

## GLME models is as follows: E(y_i) ~ g(X_i%*%beta + Z_i%*%b_i) i=1...n
## X_i is n_i by p design matrix
## beta is p by regression coefficients
## Z_i is n_i by q random design matrix (usually a subset of X_i)
## b_i is q by 1 random effects, st b_i ~ N_q(0,D), D <- M%*%t(M)
## m is cholesky decomposition (M is lower triangular) of D, we use D since elements in D are constrained
## phi are the other parameters in the distribution of [y_i| b_i]
## thetafun 
## bfun
## bfun_grad
## bfun_hess
## cfun

## Y_{ij} indpendent of Y_{i'j'} for i,i',j,j'
## log densty is as follows:
## log (f(y_{ij}|b_{ij};beta,phi,m) = (y_ij theta_ij - bfun(theta_{ij})/(phi/k_{ij}) + c(y,phi)
## theta_{ij} = thetafun(eta_{ij}), where eta_{ij} = row_j(X_i%*%beta + Z_i%*%b_i)
## E(y_{ij}|b_{ij}) = bfun(theta_{ij})
## Var(y_{ij}|b_{ij}) = phi/k_{ij} bfun_hess(theta_{ij})


** Arguments
** beta(p), regression coefficients
** m(q*(q+1)/2), vech(M), where M is cholesky decomposition of D=M%*%t(M), vech stack lower triangular portion of lower tiangular matrix
** theta = (beta,m)
** phi, other parameters in response model
** y(sum(ni)), (vec(y_1)^T,...vec(y_n)^T)^T, response times times
** X(sum(ni),p), (vec(X_1)^T,...vec(X_n)^T)^T, design matrix for fixed effects
** Z(sum(ni),q), (vec(Z_1)^T,...vec(Z_n)^T)^T, design matrix for random effects
** k(sum(ni)), (k_{11}..k_{1ni_1},k_{21},...k_{n ni_n}, dispersion matrix

** n, total number of subjects
** p, number of covariates
** ni(n), number or repeated/response observations per subject

** nsim, size of monte carlo em sample
** fileb, file name of file containing monte carlo em sample of random effects
** each row is a sample of length ni by p equal to (vec(b_{11}),...,vec(b_{1ni_1}),vec(b_{21}),...,vec(b_{n{ni_n}})) 

** functions defined above
** thetafun, 
** bfun,
** bfun_grad
** bfun_hess
** cfun

** Returned Parameters
** out, scalar with of value of log likelihood, gradient, and hessian
*/

#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include "arms.c"
#include "lasso.c"

// structure for general generalized linear regression model
typedef struct data_glmme 
{ 
  double *theta;
  double *phi;
  double *y;
  double *X;
  double *Z;
  double *k;
  double *b;
  double *bj;

  int *n;
  int *p;
  int *q;
  int *ni;
  int *i;
  int *j;

  int *nsim;
  char **fileb;

  double (*conanfun)(double x);
  double (*conanfun_grad)(double x);
  double (*conanfun_hess)(double x);
  double (*bfun)(double x);
  double (*bfun_grad)(double x);
  double (*bfun_hess)(double x);
  double (*cfun)(double x, double *phi);

  double *bmean;
  double *bvar;

  double *penalty;
  int *groupindex;
  int *numgroup;
  int *penalize;

  double *kronbitZiJqarray;
  double *etamat;

} data_glmme ;

// glmmeq1_fun,glmmeq1_grad,glmmeq1_hess calculate function, gradient and hessian of oberved data log likelihood of unpenalized portion of GLMM
 
void glmmeq1_fun(double *theta, double *phi, double *y, double *X, double *Z, double *k, int *n, int *p, int *q, int *ni, 
		 int *nsim, char **fileb, double (*conanfun)(double x), double (*conanfun_grad)(double x), double (*conanfun_hess)(double x),
		 double (*bfun)(double x), double (*bfun_grad)(double x), double (*bfun_hess)(double x), double (*cfun)(double x, double *phi), 
		 double *out) 
{

  // some preliminaries
  //printf("phi=%f\n",*phi);

  int i,j,l,one[1],sub,subin;
  one[0] = 1;

  double beta[*p],m[*q*(*q+1)/2];
  for (i=0;i<*p;i++)  
    beta[i] = theta[i];
  for (i=*p;i<*p+*q*(*q+1)/2;i++)  
    m[i-*p] = theta[i];

  // lets open MCMC sample of random effects
  //double bsimall[*nsim**n**q];
  //return;
  double *bsimall;
  // this has to be fixed
  bsimall = (double *) malloc(*nsim**n**q*sizeof(double));
  FILE *f;
 
  f = fopen(*fileb,"r");
  if (f == NULL) 
    printf("Error can't open file");
  i=0;
  while(!feof(f))  {
    fscanf(f,"%lf",&bsimall[i]);
    i++;
  }
  fclose(f);
  //unvecbyrow(*nsim,*n**q,bsimall,bsimallmat);
  // l(nsim),sub(subject),j(p,q)

  double logliki[*n],eta,conan,tmp,bit[NMAX][NMAX],kronbitZi[NMAX][NMAX],Jq[NMAX][NMAX],kronbitZiJq[NMAX][NMAX];
  double Zi[NMAX][NMAX];


  dupltmat(*q,Jq);
  setvectorto0(*n,logliki);
  for (l=0;l<*nsim;l++)  {
    for (sub=0;sub<*n;sub++)  {
      for (j=0;j<*q;j++)  {  
	for (subin=0;subin<ni[sub];subin++)  {
	  Zi[subin][j] = Z[(sub==0) ? subin+ni[sub]*j : sum_int(sub,ni)**q + (subin+ni[sub]*j)];
	}
	bit[0][j] = bsimall[l**n**q + (j+sub**q)];
      }
      kronecker(1,*q,ni[sub],*q,bit,Zi,kronbitZi); 
      matrix_mult(ni[sub],*q**q,*q*(*q+1)/2,kronbitZi,Jq,kronbitZiJq);
      for (subin=0;subin<ni[sub];subin++)  {
	for (j=0,eta=0.0;j<*p;j++)  
	  eta += X[(sub==0) ? subin+ni[sub]*j : sum_int(sub,ni)**p + (subin+ni[sub]*j)]*beta[j];
	for (j=0;j<*q*(*q+1)/2;j++)  
	  eta += kronbitZiJq[subin][j]*m[j];
	//printf("pphi=%f\n",*phi);
	//printf("eta=%f\n",eta);
	conan = conanfun(eta);
	tmp = ((double) k[(sub==0) ? subin : sum_int(sub,ni) + subin])*(y[(sub==0) ? subin : sum_int(sub,ni) + subin]*conan-bfun(conan))/(*phi);
	tmp += cfun(y[(sub==0) ? subin : sum_int(sub,ni) + subin],phi);
	//printf("A=%f,B=%f,C=%f\n",(double) karray[i][subin][0],(yarray[i][subin][0]*conan-bfun(conan))/(*phi),cfun(yarray[i][subin][0],phi));
	//printf("AA=%f,BB=%f\n",yarray[i][subin][0],*phi);
	logliki[sub] += tmp/(*nsim);
	//printf("logliki=%f\n",logliki[i]);
      }
    }
  }
  out[0] = sum(*n,logliki);
  free(bsimall);
}

void glmmeq1_grad(double *theta, double *phi, double *y, double *X, double *Z, double *k, int *n, int *p, int *q, int *ni, 
		  int *nsim, char **fileb, double (*conanfun)(double x), double (*conanfun_grad)(double x), double (*conanfun_hess)(double x),
		  double (*bfun)(double x), double (*bfun_grad)(double x), double (*bfun_hess)(double x), double (*cfun)(double x, double *phi), 
		  double *out) 
{

  // some preliminaries
  int i,j,nimax,nisum,one[1],sub;
  one[0] = 1;
  double beta[*p],m[*q*(*q+1)/2];
  for (i=0;i<*p;i++)  
    beta[i] = theta[i];
  for (i=*p;i<*p+*q*(*q+1)/2;i++)  
    m[i-*p] = theta[i];

  nimax = max_int(*n,ni);
  nisum = sum_int(*n,ni);

  // lets open MCMC sample of random effects
  double *bsimall;
  // this has to be fixed
  bsimall = (double *) malloc(*nsim**n**q*sizeof(double));
  FILE *f;

  f = fopen(*fileb,"r");
  if (f == NULL) 
    printf("Error can't open file");
  i=0;
  while(!feof(f))  {
    fscanf(f,"%lf",&bsimall[i]);
    i++;
  }
  fclose(f);

  // indicators/subscript notation is
  // sim = 1..nsim
  // sub = 1..n
  // i = 1..ni[sub]
  // j=1..p or q

  int sim,subin;
  double loglikgrad[NMAX][NMAX],eta1[nimax],eta2[nimax],conan[nimax],mean[nimax],conanfungrad[nimax],tmp,bit[NMAX][NMAX];
  double kronbitZi[NMAX][NMAX],kronbitZiJq[NMAX][NMAX],Jq[NMAX][NMAX],Zi[NMAX][NMAX],tmpy;
  dupltmat(*q,Jq);
  matrix_setpartofbigmatto0(*p+*q*(*q+1)/2,*n,loglikgrad);

  for (sim=0;sim<*nsim;sim++)  {
    for (sub=0;sub<*n;sub++)  {
      for (j=0;j<*q;j++)  {  
	for (subin=0;subin<ni[sub];subin++)  {
	  Zi[subin][j] = Z[(sub==0) ? subin+ni[sub]*j : sum_int(sub,ni)**q + (subin+ni[sub]*j)];
	}
	bit[0][j] = bsimall[sim**n**q + (j+sub**q)];
      }
      kronecker(1,*q,ni[sub],*q,bit,Zi,kronbitZi); 
      matrix_mult(ni[sub],*q**q,*q*(*q+1)/2,kronbitZi,Jq,kronbitZiJq);
      for (i=0;i<ni[sub];i++)  {
	for (j=0,eta1[i]=0.0;j<*p;j++)  
	  eta1[i] += X[(sub==0) ? i+ni[sub]*j : sum_int(sub,ni)**p + (i+ni[sub]*j)]*beta[j];
	for (j=0,eta2[i]=0.0;j<*q*(*q+1)/2;j++)  
	  eta2[i] += kronbitZiJq[i][j]*m[j];
	conan[i] = conanfun(eta1[i]+eta2[i]);
	mean[i] = bfun_grad(conan[i]);
	conanfungrad[i] = conanfun_grad(conan[i]);
      }
      for (j=0;j<*p;j++)  {
	for (i=0,tmp=0.0;i<ni[sub];i++)  {
	  tmpy = (k[(sub==0) ? i : sum_int(sub,ni) + i])*(y[(sub==0) ? i : sum_int(sub,ni) + i]- mean[i])*conanfungrad[i]/(*phi);
	  tmp += X[(sub==0) ? i+ni[sub]*j : sum_int(sub,ni)**p + (i+ni[sub]*j)]*tmpy;
	}
	loglikgrad[j][sub] += tmp/(*nsim);
      }
      for (j=*p;j<*p+*q*(*q+1)/2;j++)  {
	for (i=0,tmp=0.0;i<ni[sub];i++)  {
	  tmpy = (y[(sub==0) ? i : sum_int(sub,ni) + i]-mean[i])*conanfungrad[i]/(*phi);
	  tmp += kronbitZiJq[i][j-*p]*(k[(sub==0) ? i : sum_int(sub,ni) + i])*tmpy;
	}
	loglikgrad[j][sub] += tmp/(*nsim);
      }
    }
  } 
  for (j=0;j<*p+*q*(*q+1)/2;j++)  {
    for (i=0,tmp=0.0;i<*n;i++)  
      tmp += loglikgrad[j][i];
    out[j] = tmp;
  }
  free(bsimall);
}


void glmmeq1_hess(double *theta, double *phi, double *y, double *X, double *Z, double *k, int *n, int *p, int *q, int *ni, 
		  int *nsim, char **fileb, double (*conanfun)(double x), double (*conanfun_grad)(double x), double (*conanfun_hess)(double x),
		  double (*bfun)(double x), double (*bfun_grad)(double x), double (*bfun_hess)(double x), double (*cfun)(double x, double *phi), 
		  double *out) 
{

  // some preliminaries

  int i,j,nimax,nisum,one[1],sub;
  one[0] = 1;
  double beta[*p],m[*q*(*q+1)/2];
  for (i=0;i<*p;i++)  
    beta[i] = theta[i];
  for (i=*p;i<*p+*q*(*q+1)/2;i++)  
    m[i-*p] = theta[i];

  nimax = max_int(*n,ni);
  nisum = sum_int(*n,ni);

  double *bsimall;
  bsimall = (double *) malloc(*nsim**n**q*sizeof(double));
  // lets open MCMC sample of random effects
  FILE *f;
  f = fopen(*fileb,"r");
  if (f == NULL) 
    printf("Error can't open file");
  i=0;
  while(!feof(f))  {
    fscanf(f,"%lf",&bsimall[i]);
    i++;
  }
  fclose(f);

  // indicators/subscript notation is
  // sim = 1..nsim
  // sub = 1..n
  // i = 1..ni[sub]
  // j=1..p or q

  //ps();
  //printf("nimax=%d,nisum=%d,p=%d,q=%d,n=%d\n",nimax,nisum,*p,*q,*n);
  //return;
  int sim,subin,r,s;
  double loglikhess[*p+*q*(*q+1)/2][*p+*q*(*q+1)/2],
    eta1[nimax],eta2[nimax],conan[nimax],mean[nimax],conanfungrad[nimax],tmp;
  double bfunhess[nimax],conanfunhess[nimax],bit[NMAX][NMAX],kronbitZi[NMAX][NMAX],kronbitZiJq[NMAX][NMAX],Jq[NMAX][NMAX],Zi[NMAX][NMAX],tmpy;
  //ps();
  //return;

  dupltmat(*q,Jq);
  setmatrixto0(*p+*q*(*q+1)/2,*p+*q*(*q+1)/2,loglikhess);
  for (sim=0;sim<*nsim;sim++)  {
    for (sub=0;sub<*n;sub++)  {
      for (j=0;j<*q;j++)  {  
	for (subin=0;subin<ni[sub];subin++)  {
	  Zi[subin][j] = Z[(sub==0) ? subin+ni[sub]*j : sum_int(sub,ni)**q + (subin+ni[sub]*j)];
	}
	bit[0][j] = bsimall[sim**n**q + (j+sub**q)];
      }
      kronecker(1,*q,ni[sub],*q,bit,Zi,kronbitZi); 
      matrix_mult(ni[sub],*q**q,*q*(*q+1)/2,kronbitZi,Jq,kronbitZiJq);
      for (i=0;i<ni[sub];i++)  {
	for (j=0,eta1[i]=0.0;j<*p;j++)  
	  eta1[i] += X[(sub==0) ? i+ni[sub]*j : sum_int(sub,ni)**p + (i+ni[sub]*j)]*beta[j];
	for (j=0,eta2[i]=0.0;j<*q*(*q+1)/2;j++)  
	  eta2[i] += kronbitZiJq[i][j]*m[j];
	conan[i] = conanfun(eta1[i]+eta2[i]);
	mean[i] = bfun_grad(conan[i]);
	conanfungrad[i] = conanfun_grad(conan[i]);
	bfunhess[i] = bfun_hess(conan[i]);
	conanfunhess[i] = conanfun_hess(conan[i]);
      }
      for (r=0;r<*p;r++)  {
	for (s=0;s<*p;s++)  {
	  for (i=0,tmp=0.0;i<ni[sub];i++)  {
	    tmpy = k[((sub==0)?0:sum_int(sub,ni))+i]*(bfunhess[i]*pow(conanfungrad[i],2.0)-
							  conanfunhess[i]*(y[((sub==0)?0:sum_int(sub,ni))+i]-mean[i]))/(*phi);
	    tmp += -X[((sub==0)?0:sum_int(sub,ni)**p)+(i+ni[sub]*r)]*X[((sub==0)?0:sum_int(sub,ni)**p)+(i+ni[sub]*s)]*tmpy;

	    //	    tmp += -Xarray[sub][i][r]*(karray[sub][i][0])*(bfunhess[i]*pow(conanfungrad[i],2.0)-
	    //					   conanfunhess[i]*(yarray[sub][i][0]-mean[i]))*Xarray[sub][i][s]/(*phi);
	  }
	  loglikhess[r][s] += tmp/(*nsim);
	}
	for (s=*p;s<*p+*q*(*q+1)/2;s++)  {
	  for (i=0,tmp=0.0;i<ni[sub];i++)  {
	    tmpy = k[((sub==0)?0:sum_int(sub,ni))+i]*(bfunhess[i]*pow(conanfungrad[i],2.0)-
							  conanfunhess[i]*(y[((sub==0)?0:sum_int(sub,ni))+i]-mean[i]))*kronbitZiJq[i][s-*p]/(*phi);
	    tmp += -X[((sub==0)?0:sum_int(sub,ni)**p)+(i+ni[sub]*r)]*tmpy;			
	    //	    tmp += -Xarray[sub][i][r]*(karray[sub][i][0])*(bfunhess[i]*pow(conanfungrad[i],2.0)-
	    //					   conanfunhess[i]*(yarray[sub][i][0]-mean[i]))*kronbitZiJq[i][s-*p]/(*phi);
	  }
	  loglikhess[r][s] += tmp/(*nsim);
	}
      }
      for (r=*p;r<*p+*q*(*q+1)/2;r++)  {
	for (s=0;s<*p;s++)  {
	  for (i=0,tmp=0.0;i<ni[sub];i++)  {
	    tmpy = -kronbitZiJq[i][r-*p]*(k[((sub==0)?0:sum_int(sub,ni))+i])*(bfunhess[i]*pow(conanfungrad[i],2.0)-
									      conanfunhess[i]*(y[((sub==0)?0:sum_int(sub,ni))+i]-mean[i]))/(*phi);;
	    tmp += X[((sub==0)?0:sum_int(sub,ni)**p)+(i+ni[sub]*s)]*tmpy;
	    //	    tmp += -kronbitZiJq[i][r-*p]*(karray[sub][i][0])*(bfunhess[i]*pow(conanfungrad[i],2.0)-
	    //					   conanfunhess[i]*(yarray[sub][i][0]-mean[i]))*Xarray[sub][i][s]/(*phi);
	  }
	  loglikhess[r][s] += tmp/(*nsim);
	}
	for (s=*p;s<*p+*q*(*q+1)/2;s++)  {
	  for (i=0,tmp=0.0;i<ni[sub];i++)  {
	    tmp += -kronbitZiJq[i][r-*p]*(k[((sub==0)?0:sum_int(sub,ni))+i])*(bfunhess[i]*pow(conanfungrad[i],2.0)-
							   conanfunhess[i]*(y[((sub==0)?0:sum_int(sub,ni))+i]-mean[i]))*kronbitZiJq[i][s-*p]/(*phi);
	  }
	  //printf("tmp=%f\n",tmp);
	  loglikhess[r][s] += tmp/(*nsim);
	}
      }
    }
  } 
  //print3darray(*p+*q*(*q+1)/2,*p+*q*(*q+1)/2,*n,loglikhess);
  for (r=0;r<*p+*q*(*q+1)/2;r++)  {
    for (s=0;s<*p+*q*(*q+1)/2;s++)  {
      out[r+s*(*p+*q*(*q+1)/2)] = loglikhess[r][s];
    }
  }
  free(bsimall);
}



double conanfun_lme(double x)
{
  return x;
}

double conanfun_grad_lme(double x)
{
  return 1.0;
}

double conanfun_hess_lme(double x)
{
  return 0.0;
}

double bfun_lme(double x)
{
  double out;
  out = pow(x,2.0)/2.0;
  return out;
}

double bfun_grad_lme(double x)
{
  return x;
}

double bfun_hess_lme(double x)
{
  return 1.0;
}

double cfun_lme(double x, double *phi)
{
  double out,pi;
  pi = 3.141592653589793238462643;
  //printf("phi1=%f,phi2=%f\n",*phi,phi[0]);
  out = -1.0*(pow(x,2.0)/phi[0]+log(2.0*pi*phi[0]))/2.0;
  return out;
}

// normcd calculate distribution of random effecs model condition on observed data of NLMM
// theta, sigma2, y, X, Z, n, p, q, ni as defined above
// mean is n*q vector equal to (E(b_1|y,X,Z),...,E(b_n|y,X,Z)) 
// var is a n*q^2 vector equal to (vec(Var(b_1|y,X,Z)),...,vec(Var(b_n|y,X,Z)))

void normcd(double *theta, double *sigma2, double *y, double *X, double *Z, int *n, int *p, int *q, int *ni, double *mean, double *var)  {
  

  int i,sub,subin,j;
  double beta[*p],m[*q*(*q+1)/2],M[NMAX][NMAX],tmp;
  double Dinv[*q**q],Dsub[*q**q];
  double d[*n**q],Xbeta[sum_int(*n,ni)];
  for (i=0;i<*p;i++)  
    beta[i] = theta[i];
  for (i=*p;i<*p+*q*(*q+1)/2;i++)  
    m[i-*p] = theta[i];
  ltmat(*q,m,M);

  double ZM[sum_int(*n,ni)**q];
  for (sub=0;sub<*n;sub++)  {
    for (subin=0;subin<ni[sub];subin++)  {
      for (j=0;j<*q;j++)  {
	for (i=0,tmp=0.0;i<*q;i++) {
	  tmp += Z[((sub==0)?0:sum_int(sub,ni)**q)+subin+ni[sub]*i]*M[i][j];
	}
	ZM[((sub==0)?0:sum_int(sub,ni)**q)+(subin+ni[sub]*j)] = tmp;
      }
    }
  }
  for (sub=0;sub<*n;sub++)  {
    for (i=0;i<*q;i++)  {
      for (j=0;j<*q;j++)  {
	for (subin=0,tmp=0.0;subin<ni[sub];subin++) {
	  tmp += ZM[((sub==0)?0:sum_int(sub,ni)**q)+subin+ni[sub]*i]*ZM[((sub==0)?0:sum_int(sub,ni)**q)+subin+ni[sub]*j];
	}
	Dinv[i+*q*j]=tmp/(*sigma2)+((i==j)?1:0);
      }
    }
    ginvwrapper(q,q,Dinv,Dsub);
    //vp(*q**q,Dsub,"Dsub");
    for (i=0;i<*q;i++)  {
      for (j=0;j<*q;j++)  {
	var[sub**q**q+i+*q*j]=Dsub[i+*q*j];
      }
    }
  }
  for (sub=0;sub<*n;sub++)  {
    for (subin=0;subin<ni[sub];subin++) {
      for (j=0,tmp=0;j<*p;j++)
	tmp += X[((sub==0)?0:sum_int(sub,ni)**p)+subin+ni[sub]*j]*beta[j];
      Xbeta[((sub==0)?0:sum_int(sub,ni))+subin] = tmp;
    }
  }
  //vp(sum_int(*n,ni),Xbeta,"Xbeta");
  for (sub=0;sub<*n;sub++)  {
    for (i=0;i<*q;i++)  {
      for (subin=0,tmp=0.0;subin<ni[sub];subin++) {
	tmp += ZM[((sub==0)?0:sum_int(sub,ni)**q)+(subin+ni[sub]*i)]*
	  (y[((sub==0)?0:sum_int(sub,ni))+subin]-Xbeta[((sub==0)?0:sum_int(sub,ni))+subin]);
      }
      d[sub**q+i]=tmp/(*sigma2);
    }
  }
  //vp(sum_int(*n,ni),d,"d");
  for (sub=0;sub<*n;sub++)  {
    for (i=0;i<*q;i++)  {
      for (j=0,tmp=0;j<*q;j++)  {
	tmp += var[sub**q**q+i+*q*j]*d[sub**q+j];
      }
      mean[i+*q*sub]=tmp;
    }
  }
}

// lmeq1_fun, lmeq2_grad, lmeq_hess are function value, gradient and hessian of a NLMM 
// theta, phi, y, X, Z, n, p, q, ni are defined as before. 
// bmean is n*q vector equal to (E(b_1|y,X,Z),...,E(b_n|y,X,Z)) 
// bvar is a n*q^2 vector equal to (vec(Var(b_1|y,X,Z)),...,vec(Var(b_n|y,X,Z)))

void lmeq1_fun(double *theta, double *phi, double *y, double *X, double *Z, double *bmean, double *bvar, int *n, int *p, int *q, int *ni, double *out) 
{

  int i,j,l,one[1],sub,subin;
  one[0] = 1;

  double beta[*p],m[*q*(*q+1)/2],pi;
  for (i=0;i<*p;i++)  
    beta[i] = theta[i];
  for (i=*p;i<*p+*q*(*q+1)/2;i++)  
    m[i-*p] = theta[i];

  //nimax = max_int(*n,ni);

  double eta,Zi[NMAX][NMAX],bit[NMAX][NMAX],bvari[NMAX][NMAX],kronbitZi[NMAX][NMAX],ZitZi[NMAX][NMAX],kronbitZiJq[NMAX][NMAX],Jq[NMAX][NMAX],
    Jqt[NMAX][NMAX],kronbvariZitZi[NMAX][NMAX],kronbvariZitZiJq[NMAX][NMAX],JqtkronbvariZitZiJq[NMAX][NMAX],
    *kronbitZiJqarray,*JqtkronbvariZitZiJqarray,tmp;
  kronbitZiJqarray = (double *) malloc(sum_int(*n,ni)**q*(*q+1)/2*sizeof(double));
  JqtkronbvariZitZiJqarray = (double *) malloc(*n**q*(*q+1)/2**q*(*q+1)/2*sizeof(double));



  dupltmat(*q,Jq);
  matrix_transposetomat(*q**q,*q*(*q+1)/2,Jq,Jqt);
  for(sub=0;sub<*n;sub++)  {
    for (j=0;j<*q;j++)  {  
      for (subin=0;subin<ni[sub];subin++)  {
 	Zi[subin][j] = Z[(sub==0) ? subin+ni[sub]*j : sum_int(sub,ni)**q + (subin+ni[sub]*j)];
      }
      for (l=0;l<*q;l++)  
	bvari[j][l] = bvar[sub**q**q+(j+*q*l)];
      bit[0][j] = bmean[j+sub**q];
    }
    //bmp(ni[sub],*q,Zi,"Zi");
    //bmp(1,*q,bit,"bit");
    kronecker(1,*q,ni[sub],*q,bit,Zi,kronbitZi); 
    //bmp(ni[sub],*q**q,kronbitZi,"kronbitZi");
    matrix_mult(ni[sub],*q**q,*q*(*q+1)/2,kronbitZi,Jq,kronbitZiJq);
    matrix_crossprod(ni[sub],*q,Zi,ZitZi);
    kronecker(*q,*q,*q,*q,bvari,ZitZi,kronbvariZitZi);
    matrix_mult(*q**q,*q**q,*q*(*q+1)/2,kronbvariZitZi,Jq,kronbvariZitZiJq);
    matrix_mult(*q*(*q+1)/2,*q**q,*q*(*q+1)/2,Jqt,kronbvariZitZiJq,JqtkronbvariZitZiJq);
    //bmp(ni[sub],*q*(*q+1)/2,kronbitZiJq,"kronbitZiJq");
    //bmp(*q*(*q+1)/2,*q*(*q+1)/2,JqtkronbvariZitZiJq,"JqtkronbvariZitZiJq");
    for (j=0;j<*q*(*q+1)/2;j++)  {  
      for (l=0;l<ni[sub];l++)  {
	kronbitZiJqarray[(sub==0)? l+ni[sub]*j : sum_int(sub,ni)**q*(*q+1)/2 + (l+ni[sub]*j)] = kronbitZiJq[l][j];
      }
      for (l=0;l<*q*(*q+1)/2;l++)  {
	JqtkronbvariZitZiJqarray[sub*(*q*(*q+1)/2)*(*q*(*q+1)/2) + (j+(*q*(*q+1)/2)*l)] = JqtkronbvariZitZiJq[j][l];
      }
    }
  }
  //ps();
  //return;
  for (sub=0,out[0]=0.0;sub<*n;sub++)  {
    for (subin=0;subin<ni[sub];subin++)  {
      for (j=0,eta=0.0;j<*p;j++)  
	eta += X[(sub==0) ? subin+ni[sub]*j : sum_int(sub,ni)**p + (subin+ni[sub]*j)]*beta[j];
      //printf("eta=%f\n",eta);
      for (j=0;j<*q*(*q+1)/2;j++)  
	eta += kronbitZiJqarray[(sub==0) ? subin+ni[sub]*j : sum_int(sub,ni)**q*(*q+1)/2 + (subin+ni[sub]*j)]*m[j];
      //printf("eta=%f\n",eta);
      out[0] += pow(y[(sub==0) ? subin : sum_int(sub,ni) + subin]-eta,2.0);
      //printf("out[0]=%f\n",out[0]);
    }
  }
  //printf("out[0]=%f\n",out[0]);
  for (sub=0;sub<*n;sub++)  {
    for (j=0;j<*q*(*q+1)/2;j++)  {
      for (l=0,tmp=0.0;l<*q*(*q+1)/2;l++)  
	tmp += JqtkronbvariZitZiJqarray[sub*(*q*(*q+1)/2)*(*q*(*q+1)/2) + (j+(*q*(*q+1)/2)*l)]*m[l];
      out[0] += m[j]*tmp;
    }
  }

  double sigma2,nisum;
  sigma2=phi[0];
  pi = 3.141592653589793238462643;
  nisum = (double) sum_int(*n,ni);
  out[0] = -(nisum/2.0)*log(2.0*pi)-(nisum/2.0)*log(sigma2)-out[0]/(2.0*sigma2);
  free(kronbitZiJqarray);
  free(JqtkronbvariZitZiJqarray);
}

void lmeq1_grad(double *theta, double *phi, double *y, double *X, double *Z, double *bmean, double *bvar, int *n, int *p, int *q, int *ni, double *out) 
{


  int i,j,l,nimax,nisum,one[1],sub,subin;
  one[0] = 1;

  double beta[*p],m[*q*(*q+1)/2];
  for (i=0;i<*p;i++)  
    beta[i] = theta[i];
  for (i=*p;i<*p+*q*(*q+1)/2;i++)  
    m[i-*p] = theta[i];

  nimax = max_int(*n,ni);
  nisum = sum_int(*n,ni);

  //print3darray(*n,nimax,*q,Zarray);

  double eta,Zi[NMAX][NMAX],bit[NMAX][NMAX],bvari[NMAX][NMAX],kronbitZi[NMAX][NMAX],ZitZi[NMAX][NMAX],kronbitZiJq[NMAX][NMAX],Jq[NMAX][NMAX],
    Jqt[NMAX][NMAX],kronbvariZitZi[NMAX][NMAX],kronbvariZitZiJq[NMAX][NMAX],JqtkronbvariZitZiJq[NMAX][NMAX],
    *kronbitZiJqarray,*JqtkronbvariZitZiJqarray,tmp;
  kronbitZiJqarray = (double *) malloc(nisum**q*(*q+1)/2*sizeof(double));
  JqtkronbvariZitZiJqarray = (double *) malloc(*n**q*(*q+1)/2**q*(*q+1)/2*sizeof(double));

  //ivp(*n,ni,"ni");
  dupltmat(*q,Jq);

  matrix_transposetomat(*q**q,*q*(*q+1)/2,Jq,Jqt);
  for(sub=0;sub<*n;sub++)  {
    for (j=0;j<*q;j++)  {  
      for (subin=0;subin<ni[sub];subin++)  {
 	Zi[subin][j] = Z[(sub==0) ? subin+ni[sub]*j : sum_int(sub,ni)**q + (subin+ni[sub]*j)];
      }
      for (l=0;l<*q;l++)  
	bvari[j][l] = bvar[sub**q**q+(j+*q*l)];
      bit[0][j] = bmean[j+sub**q];
    }
    //bmp(ni[sub],*q,Zi,"Zi");
    //bmp(1,*q,bit,"bit");
    kronecker(1,*q,ni[sub],*q,bit,Zi,kronbitZi); 
    //bmp(ni[sub],*q**q,kronbitZi,"kronbitZi");
    matrix_mult(ni[sub],*q**q,*q*(*q+1)/2,kronbitZi,Jq,kronbitZiJq);
    matrix_crossprod(ni[sub],*q,Zi,ZitZi);
    kronecker(*q,*q,*q,*q,bvari,ZitZi,kronbvariZitZi);
    matrix_mult(*q**q,*q**q,*q*(*q+1)/2,kronbvariZitZi,Jq,kronbvariZitZiJq);
    matrix_mult(*q*(*q+1)/2,*q**q,*q*(*q+1)/2,Jqt,kronbvariZitZiJq,JqtkronbvariZitZiJq);
    //bmp(ni[sub],*q*(*q+1)/2,kronbitZiJq,"kronbitZiJq");
    //bmp(*q*(*q+1)/2,*q*(*q+1)/2,JqtkronbvariZitZiJq,"JqtkronbvariZitZiJq");
    for (j=0;j<*q*(*q+1)/2;j++)  {  
      for (l=0;l<ni[sub];l++)  {
	kronbitZiJqarray[(sub==0)? l+ni[sub]*j : sum_int(sub,ni)**q*(*q+1)/2 + (l+ni[sub]*j)] = kronbitZiJq[l][j];
      }
      for (l=0;l<*q*(*q+1)/2;l++)  {
	JqtkronbvariZitZiJqarray[sub*(*q*(*q+1)/2)*(*q*(*q+1)/2) + (j+(*q*(*q+1)/2)*l)] = JqtkronbvariZitZiJq[j][l];
      }
    }
  }

  for (j=0;j<*p+*q*(*q+1)/2;j++)  
    out[j]=0;
  double etamat[*n][nimax];
 
  for (sub=0;sub<*n;sub++)  {
    for (subin=0;subin<ni[sub];subin++)  {
      for (j=0,eta=0.0;j<*p;j++)  
	eta += X[(sub==0) ? subin+ni[sub]*j : sum_int(sub,ni)**p + (subin+ni[sub]*j)]*beta[j];
      for (j=0;j<*q*(*q+1)/2;j++)  
	eta += kronbitZiJqarray[(sub==0) ? subin+ni[sub]*j : sum_int(sub,ni)**q*(*q+1)/2 + (subin+ni[sub]*j)]*m[j];
      etamat[sub][subin]=eta;
    }
  }
  //printf("eta=%f\n",eta);
  for (sub=0;sub<*n;sub++)  {
    for (j=0;j<*p;j++)  {
      for (subin=0,tmp=0;subin<ni[sub];subin++)  {
	tmp += X[(sub==0) ? subin+ni[sub]*j : sum_int(sub,ni)**p + (subin+ni[sub]*j)]*(y[(sub==0) ? subin : sum_int(sub,ni) + subin]-etamat[sub][subin]);
      }
      //printf("sub=%d,j=%d,tmp=%f\n",sub,j,tmp);
      out[j] += tmp;
    }
    for (j=0;j<*q*(*q+1)/2;j++)  {
      for (subin=0,tmp=0;subin<ni[sub];subin++)  {
	tmp += kronbitZiJqarray[(sub==0) ? subin+ni[sub]*j : sum_int(sub,ni)**q*(*q+1)/2 + 
				(subin+ni[sub]*j)]*(y[(sub==0) ? subin : sum_int(sub,ni) + subin]-etamat[sub][subin]);
      }
      out[j+*p] += tmp;
    }
  }
  //vp(*p+*q*(*q+1)/2,out,"out");
  //for (j=0;j<*p+*q*(*q+1)/2;j++)  
  //x  out[j]=0;
  for (sub=0;sub<*n;sub++)  {
    for (j=0;j<*q*(*q+1)/2;j++)  {
      for (i=0,tmp=0;i<*q*(*q+1)/2;i++)  {
	tmp += JqtkronbvariZitZiJqarray[sub*(*q*(*q+1)/2)*(*q*(*q+1)/2) + (j+(*q*(*q+1)/2)*i)]*m[i];
      }
      out[j+*p] -= tmp;
    }
  }
  //vp(*p+*q*(*q+1)/2,out,"out");
  //printf("out=%f\n",out[0]);
  double sigma2;
  sigma2=phi[0];
  for (j=0;j<*p+*q*(*q+1)/2;j++)  {
    out[j]= out[j]/(sigma2);
  }

  free(kronbitZiJqarray);
  free(JqtkronbvariZitZiJqarray);

}

void lmeq1_hess(double *theta, double *phi, double *y, double *X, double *Z, double *bmean, double *bvar, int *n, int *p, int *q, int *ni, double *out) 
{

  int i,j,l,nimax,nisum,one[1],sub,subin;
  one[0] = 1;

  double beta[*p],m[*q*(*q+1)/2];
  for (i=0;i<*p;i++)  
    beta[i] = theta[i];
  for (i=*p;i<*p+*q*(*q+1)/2;i++)  
    m[i-*p] = theta[i];

  nimax = max_int(*n,ni);
  nisum = sum_int(*n,ni);

  //print3darray(*n,nimax,*q,Zarray);

  double Zi[NMAX][NMAX],bit[NMAX][NMAX],bvari[NMAX][NMAX],kronbitZi[NMAX][NMAX],ZitZi[NMAX][NMAX],kronbitZiJq[NMAX][NMAX],Jq[NMAX][NMAX],
    Jqt[NMAX][NMAX],kronbvariZitZi[NMAX][NMAX],kronbvariZitZiJq[NMAX][NMAX],JqtkronbvariZitZiJq[NMAX][NMAX],
    *kronbitZiJqarray,*JqtkronbvariZitZiJqarray,tmp;
  kronbitZiJqarray = (double *) malloc(nisum**q*(*q+1)/2*sizeof(double));
  JqtkronbvariZitZiJqarray = (double *) malloc(*n**q*(*q+1)/2**q*(*q+1)/2*sizeof(double));

  
  //ivp(*n,ni,"ni");
  dupltmat(*q,Jq);
  matrix_transposetomat(*q**q,*q*(*q+1)/2,Jq,Jqt);
  for(sub=0;sub<*n;sub++)  {
    for (j=0;j<*q;j++)  {  
      for (subin=0;subin<ni[sub];subin++)  {
 	Zi[subin][j] = Z[(sub==0) ? subin+ni[sub]*j : sum_int(sub,ni)**q + (subin+ni[sub]*j)];
      }
      for (l=0;l<*q;l++)  
	bvari[j][l] = bvar[sub**q**q+(j+*q*l)];
      bit[0][j] = bmean[j+sub**q];
    }
    //bmp(ni[sub],*q,Zi,"Zi");
    //bmp(1,*q,bit,"bit");
    kronecker(1,*q,ni[sub],*q,bit,Zi,kronbitZi); 
    //bmp(ni[sub],*q**q,kronbitZi,"kronbitZi");
    matrix_mult(ni[sub],*q**q,*q*(*q+1)/2,kronbitZi,Jq,kronbitZiJq);
    matrix_crossprod(ni[sub],*q,Zi,ZitZi);
    kronecker(*q,*q,*q,*q,bvari,ZitZi,kronbvariZitZi);
    matrix_mult(*q**q,*q**q,*q*(*q+1)/2,kronbvariZitZi,Jq,kronbvariZitZiJq);
    matrix_mult(*q*(*q+1)/2,*q**q,*q*(*q+1)/2,Jqt,kronbvariZitZiJq,JqtkronbvariZitZiJq);
    //bmp(ni[sub],*q*(*q+1)/2,kronbitZiJq,"kronbitZiJq");
    //bmp(*q*(*q+1)/2,*q*(*q+1)/2,JqtkronbvariZitZiJq,"JqtkronbvariZitZiJq");
    for (j=0;j<*q*(*q+1)/2;j++)  {  
      for (l=0;l<ni[sub];l++)  {
	kronbitZiJqarray[(sub==0)? l+ni[sub]*j : sum_int(sub,ni)**q*(*q+1)/2 + (l+ni[sub]*j)] = kronbitZiJq[l][j];
      }
      for (l=0;l<*q*(*q+1)/2;l++)  {
	JqtkronbvariZitZiJqarray[sub*(*q*(*q+1)/2)*(*q*(*q+1)/2) + (j+(*q*(*q+1)/2)*l)] = JqtkronbvariZitZiJq[j][l];
      }
    }
  }
  double output[NMAX][NMAX];
  matrix_setpartofbigmatto0(*p+*q*(*q+1)/2,*p+*q*(*q+1)/2,output);
  for (sub=0;sub<*n;sub++)  {
    for (j=0;j<*p;j++)  {
      for (i=0;i<*p;i++)  {
	for (subin=0,tmp=0;subin<ni[sub];subin++)  {
	  tmp += X[(sub==0) ? subin+ni[sub]*j : sum_int(sub,ni)**p 
		   + (subin+ni[sub]*j)]*X[(sub==0) ? subin+ni[sub]*i : sum_int(sub,ni)**p + (subin+ni[sub]*i)];
	}
	output[j][i] += tmp;
      }
    }
  }
  for (sub=0;sub<*n;sub++)  {
    for (j=0;j<*p;j++)  {
      for (i=0;i<*q*(*q+1)/2;i++)  {
	for (subin=0,tmp=0;subin<ni[sub];subin++)  {
	  tmp += X[(sub==0) ? subin+ni[sub]*j : sum_int(sub,ni)**p 
		   + (subin+ni[sub]*j)]*kronbitZiJqarray[(sub==0)? subin+ni[sub]*i : sum_int(sub,ni)**q*(*q+1)/2 + (subin+ni[sub]*i)];
	  //tmp += Xarray[sub][subin][j]*kronbitZiJqarray[sub][subin][i];
	}
	output[j][i+*p] += tmp;
      }
    }
    for (j=0;j<*q*(*q+1)/2;j++)  {
      for (i=0;i<*p;i++)  {
	for (subin=0,tmp=0;subin<ni[sub];subin++)  {
	  tmp += X[(sub==0) ? subin+ni[sub]*i : sum_int(sub,ni)**p 
		   + (subin+ni[sub]*i)]*kronbitZiJqarray[(sub==0)? subin+ni[sub]*j : sum_int(sub,ni)**q*(*q+1)/2 + (subin+ni[sub]*j)];
	  //tmp += kronbitZiJqarray[sub][subin][j]*Xarray[sub][subin][i];
	}
	output[j+*p][i] += tmp;
      }
    }
  }
  for (sub=0;sub<*n;sub++)  {
    for (j=0;j<*q*(*q+1)/2;j++)  {
      for (i=0;i<*q*(*q+1)/2;i++)  {
	for (subin=0,tmp=0;subin<ni[sub];subin++)  {
	  tmp += kronbitZiJqarray[(sub==0)? subin+ni[sub]*j : sum_int(sub,ni)**q*(*q+1)/2 
				  + (subin+ni[sub]*j)]*kronbitZiJqarray[(sub==0)? subin+ni[sub]*i : sum_int(sub,ni)**q*(*q+1)/2 + (subin+ni[sub]*i)];
	  //tmp += kronbitZiJqarray[sub][subin][j]*kronbitZiJqarray[sub][subin][i];
	}
	output[j+*p][i+*p] += tmp+JqtkronbvariZitZiJqarray[sub*(*q*(*q+1)/2)*(*q*(*q+1)/2) + (j+(*q*(*q+1)/2)*i)];
	//output[j+*p][i+*p] += tmp+JqtkronbvariZitZiJqarray[sub][j][i];
      }
    }
  }
  //printf("out=%f\n",out[0]);
  double sigma2;
  sigma2=phi[0];
  for (j=0;j<*p+*q*(*q+1)/2;j++)  {
    for (i=0;i<*p+*q*(*q+1)/2;i++)  {
      out[j+(*p+*q*(*q+1)/2)*i]= -output[j][i]/(sigma2);
    }
  }
  free(kronbitZiJqarray);
  free(JqtkronbvariZitZiJqarray);
}




