
source("lme_auxfun.r")
library(HI)

## 02/17/2008 function which perform linear mixed effects model
## functions below which call these C functions
## to dynamically load function use R CMD SHLIB to create .so/.so and and dyn.load to load

## LME models is as follows: y_i ~ X_i%*%beta + Z_i%*%b_i + e_i, i=1...N
## X_i is n_i by p design matrix
## beta is p by regression coefficients
## Z_i is n_i by q random design matrix (usually a subset of X_i)
## b_i is q by 1 random effects, st b_i ~ N_q(0,D), D <- M%*%t(M)
## M is singular cholesky decomposition (M is lower triangular) of D, we use D since elements in D are unconstrained
## e_i ~ N_{n_i}(0,I_{n_i}sigma2)

## D is positive semi-definite
## We express the density of a singular normal (SN) distribution as follows
## b_i ~ SN_q(0,M%*%t(M))
## f(b_i|M) = (2\pi)^{-q/2} \sum_{l=1}^q (\log m_{ll}^{m_{ll} != 0}) exp(-t(b_i)%*%t(M^{+})%*%M^{+}%*%b_i/2)
## where M^{+} is the Moore-Penrose inverse of M

## in the following functions the arguments are as follows
## y is n by 1 list where each element of y is n_i by 1 reponse vector corresponding to subject i
## X is a n by 1 list where each element of X is a n_i by p design matrix of fixed effects corresponding to subject i
## Z is a n by 1 list where each element of X is a n_i by q design matrix of random effects the corresponding to subject i
## sigma2, scalar corresponding to variance of reponses conditional on random effects
## beta, p by 1 vector of fixed effects
## M, q by q covariance matrix of random effects

## q1 <- sum_{i=1}^n dnorm(y_i,X_i%*%beta+Z_i%*%b,I_ni sigma2)
## q2 <- sum dnorm(b_i,0,M%*%t(M))


## red.r calculates the E(b|y,X) and Var(b|y,X)

red.r <- function(theta,sigma2,y,X,Z,n,p,q)  {
  beta <- theta[1:p,1,drop=FALSE]
  m <- theta[(p+1):(p+q*(q+1)/2),1,drop=FALSE]
  M <- ltmat(m)
  fun1 <- function(i)  {
    out <- solve(t(Z[[i]]%*%M)%*%(Z[[i]]%*%M)/sigma2+diag(q))
    out
  }
  fun2 <- function(i)  {
    out <- t(Z[[i]]%*%M)%*%(y[[i]]-X[[i]]%*%beta)/sigma2
    out
  }
  out1 <- lapply(1:n,fun1)
  out2 <- lapply(1:n,fun2)
  out3 <- lapply(1:n,function(i) out1[[i]]%*%out2[[i]])
  out <- list(mean=out3,var=out1)
  ##out <- list(mean=out2,var=out1)
  out
}

## lmeq1.function.r, lmeq1.grad.r, lmeq1.hess.r calculates the q function, gradient, and hessian w/ respect to theta
## of the likelihood of linear mixed effects model


lmeq1.function.r <- function(theta,sigma2,y,X,Z,b,n,p,q)  {
  b.mean <- b$mean
  b.var <- b$var
  beta <- theta[1:p,1,drop=FALSE]
  m <- theta[(p+1):(p+q*(q+1)/2),1,drop=FALSE]
  M <- ltmat(m)
  fun <- function(i)  {
    ni <- nrow(X[[i]])
    out <- -ni/2*log(2*pi)-ni/2*log(sigma2)-crossprod(y[[i]]-X[[i]]%*%beta-Z[[i]]%*%M%*%b.mean[[i]])/2/sigma2
    ##out <- crossprod(y[[i]]-X[[i]]%*%beta-Z[[i]]%*%M%*%b.mean[[i]])
    out <- out-sum(diag(Z[[i]]%*%M%*%b.var[[i]]%*%t(Z[[i]]%*%M)))/2/sigma2
    ##out <- sum(diag(Z[[i]]%*%M%*%b.var[[i]]%*%t(Z[[i]]%*%M)))
    out <- out-q/2*log(2*pi)-crossprod(b.mean[[i]])/2-sum(diag(b.var[[i]]))/2
    out
  }
  out <- sum(sapply(1:n,fun))
  out
}

lmeq1.grad.r <- function(theta,sigma2,y,X,Z,b,n,p,q)  {
  b.mean <- b$mean
  b.var <- b$var
  beta <- theta[1:p,1,drop=FALSE]
  m <- theta[(p+1):(p+q*(q+1)/2),1,drop=FALSE]
  M <- ltmat(m)
  Jq <- dupltmat(q)
  Kqq <- commutmat(q,q)
  fun <- function(i)  {
    out <- t(cbind(X[[i]],kronecker(t(b.mean[[i]]),Z[[i]])%*%Jq))%*%(y[[i]]-X[[i]]%*%beta-kronecker(t(b.mean[[i]]),Z[[i]])%*%Jq%*%m)/sigma2
    out <- out-rbind(matrix(0,p,1),t(Jq)%*%kronecker(b.var[[i]],crossprod(Z[[i]]))%*%Jq%*%m/sigma2)
    ##out <- t(cbind(X[[i]],kronecker(t(b.mean[[i]]),Z[[i]])%*%Jq))%*%(y[[i]]-X[[i]]%*%beta-kronecker(t(b.mean[[i]]),Z[[i]])%*%Jq%*%m)
    ##out <- rbind(matrix(0,p,1),t(Jq)%*%kronecker(b.var[[i]],crossprod(Z[[i]]))%*%Jq%*%m)
    ##browser()
    out
  }
  out <- matrix(apply(sapply(1:n,fun),1,sum),,1)
  out
}

lmeq1.hess.r <- function(theta,sigma2,y,X,Z,b,n,p,q)  {
  b.mean <- b$mean
  b.var <- b$var
  Jq <- dupltmat(q)
  fun <- function(i)  {
    out <- -t(cbind(X[[i]],kronecker(t(b.mean[[i]]),Z[[i]])%*%Jq))%*%cbind(X[[i]],kronecker(t(b.mean[[i]]),Z[[i]])%*%Jq)/sigma2
    out  <- out-blockdiag(matrix(0,p,p),t(Jq)%*%kronecker(b.var[[i]],crossprod(Z[[i]]))%*%Jq/sigma2)
    out
  }
  out <- apply(sapply(1:n,fun),1,sum)
  out <- matrix(out,p+q*(q+1)/2,p+q*(q+1)/2)
  out
}

## Internal C versions of models above

lmeq1.function <- function(theta,sigma2,y,X,Z,b,n,p,q)  {
  b.mean <- b$mean
  b.var <- b$var
  ni <- sapply(y,nrow)
  out <- .C("lmeq1_fun",as.double(theta),as.double(sigma2),as.double(unlist(y)),as.double(unlist(X)),as.double(unlist(Z)),
            as.double(unlist(b.mean)),as.double(unlist(b.var)),as.integer(n),as.integer(p),as.integer(q),
            as.integer(unlist(ni)),out=double(length=1))$out
  out
}
lmeq1.function <- lmeq1.function.r 
lmeq1.grad <- function(theta,sigma2,y,X,Z,b,n,p,q)  {
  b.mean <- b$mean
  b.var <- b$var
  ni <- sapply(y,nrow)
  out <- .C("lmeq1_grad",as.double(theta),as.double(sigma2),as.double(unlist(y)),as.double(unlist(X)),as.double(unlist(Z)),
            as.double(unlist(b.mean)),as.double(unlist(b.var)),as.integer(n),as.integer(p),as.integer(q),
            as.integer(unlist(ni)),out=double(length=p+q*(q+1)/2))$out
  out <- matrix(out,,1)
  out
}
lmeq1.hess <- function(theta,sigma2,y,X,Z,b,n,p,q)  {
  b.mean <- b$mean
  b.var <- b$var
  ni <- sapply(y,nrow)
  out <- .C("lmeq1_hess",as.double(theta),as.double(sigma2),as.double(unlist(y)),as.double(unlist(X)),as.double(unlist(Z)),
            as.double(unlist(b.mean)),as.double(unlist(b.var)),as.integer(n),as.integer(p),as.integer(q),
            as.integer(unlist(ni)),out=double(length=(p+q*(q+1)/2)^2))$out
  out <- matrix(out,(p+q*(q+1)/2),(p+q*(q+1)/2))
  out
}


lmeq1 <- structure(lmeq1.function,gradient=lmeq1.grad,hessian=lmeq1.hess)

## Profiled versions of functions above

lmeq1prof.function.r <- function(theta,y,X,Z,b,n,p,q)  {
  b.mean <- b$mean
  b.var <- b$var
  ni <- sapply(X,nrow)
  SS <- -2*(lmeq1.function(theta,1,y,X,Z,b,n,p,q)+sum(ni)/2*log(2*pi))
  out <- -(sum(ni)/2)*log(SS)
  out
}
lmeq1prof.grad.r <- function(theta,y,X,Z,b,n,p,q)  {
  b.mean <- b$mean
  b.var <- b$var
  ni <- sapply(X,nrow)
  SS <- -2*(lmeq1.function(theta,1,y,X,Z,b,n,p,q)+sum(ni)/2*log(2*pi))
  out <- sum(ni)*lmeq1.grad(theta,1,y,X,Z,b,n,p,q)/SS
  out
}
lmeq1prof.hess.r <- function(theta,y,X,Z,b,n,p,q)  {
  b.mean <- b$mean
  b.var <- b$var
  ni <- sapply(X,nrow)
  SS <- -2*(lmeq1.function(theta,1,y,X,Z,b,n,p,q)+sum(ni)/2*log(2*pi))
  mat1 <- lmeq1.hess(theta,1,y,X,Z,b,n,p,q)
  mat2 <- lmeq1.grad(theta,1,y,X,Z,b,n,p,q)
  out <- sum(ni)*mat1/SS+sum(ni)*2*crossprod(t(mat2))/(SS)^2
  out
}
lmeq1prof <- structure(lmeq1prof.function.r,gradient=lmeq1prof.grad.r,hessian=lmeq1prof.hess.r)
lmeq1prof.function <- function(theta,y,X,Z,b,n,p,q)  {
  b.mean <- b$mean
  b.var <- b$var
  ni <- sapply(y,nrow)
  out <- .C("lmeq1prof_fun",as.double(theta),as.double(unlist(y)),as.double(unlist(X)),as.double(unlist(Z)),
            as.double(unlist(b.mean)),as.double(unlist(b.var)),as.integer(n),as.integer(p),as.integer(q),as.integer(unlist(ni)),
            out=double(length=1))$out
  out
}
lmeq1prof.grad <- function(theta,y,X,Z,b,n,p,q)  {
  b.mean <- b$mean
  b.var <- b$var
  ni <- sapply(y,nrow)
  out <- .C("lmeq1prof_grad",as.double(theta),as.double(unlist(y)),as.double(unlist(X)),as.double(unlist(Z)),
            as.double(unlist(b.mean)),as.double(unlist(b.var)),as.integer(n),as.integer(p),as.integer(q),as.integer(unlist(ni)),
            out=double(length=(p+q*(q+1)/2)))$out
  out
}
lmeq1prof.hess <- function(theta,y,X,Z,b,n,p,q)  {
  b.mean <- b$mean
  b.var <- b$var
  ni <- sapply(y,nrow)
  out <- .C("lmeq1prof_hess",as.double(theta),as.double(unlist(y)),as.double(unlist(X)),as.double(unlist(Z)),
            as.double(unlist(b.mean)),as.double(unlist(b.var)),as.integer(n),as.integer(p),as.integer(q),as.integer(unlist(ni)),
            out=double(length=(p+q*(q+1)/2)^2))$out
  out <- matrix(out,(p+q*(q+1)/2),(p+q*(q+1)/2))
  out
}
lmeq1prof <- structure(lmeq1prof.function,gradient=lmeq1prof.grad,hessian=lmeq1prof.hess)

## compute next iterate of Newton Rhapson, LQA, and LLA algorithms

nexttheta.mle.lmeq1 <- function(theta,sigma2,y,X,Z,b,n,p,q)  {
  b.mean <- b$mean
  b.var <- b$var
  ni <- sapply(y,nrow)
  k <- lapply(ni,function(x) rep(1,x))
  out <- .C("nexttheta_mle_lmeq1",as.double(theta),as.double(sigma2),as.double(unlist(y)),as.double(unlist(X)),as.double(unlist(Z)),
            as.double(unlist(b.mean)),as.double(unlist(b.var)),as.integer(n),as.integer(p),as.integer(q),as.integer(unlist(ni)),
            out=double(length=(p+q*(q+1)/2)))$out
  out <- matrix(out,(p+q*(q+1)/2),1)
  out
}
nexttheta.grouplqa.scad.lmeq1 <- function(theta,sigma2,y,X,Z,b,n,p,q,penalize,penalty,groupindex,numgroup)  {
  b.mean <- b$mean
  b.var <- b$var
  ni <- sapply(y,nrow)
  out <- .C("nexttheta_grouplqa_scad_lmeq1",as.double(theta),as.double(sigma2),as.double(unlist(y)),as.double(unlist(X)),as.double(unlist(Z)),
            as.double(unlist(b.mean)),as.double(unlist(b.var)),as.integer(n),as.integer(p),as.integer(q),as.integer(unlist(ni)),
            as.double(penalize),as.double(penalty),as.integer(groupindex-1),as.integer(numgroup),out=double(length=(p+q*(q+1)/2)))$out
  out <- matrix(out,(p+q*(q+1)/2),1)
  out
}
nexttheta.grouplla.scad.lmeq1 <- function(theta,sigma2,y,X,Z,b,n,p,q,penalize,penalty,groupindex,numgroup)  {
  b.mean <- b$mean
  b.var <- b$var
  ni <- sapply(y,nrow)
  out <- .C("nexttheta_grouplla_scad_lmeq1",as.double(theta),as.double(sigma2),as.double(unlist(y)),as.double(unlist(X)),as.double(unlist(Z)),
            as.double(unlist(b.mean)),as.double(unlist(b.var)),as.integer(n),as.integer(p),as.integer(q),as.integer(unlist(ni)),
            as.double(penalize),as.double(penalty),as.integer(groupindex-1),as.integer(numgroup),out=double(length=(p+q*(q+1)/2)))$out
  out <- matrix(out,(p+q*(q+1)/2),1)
  out
}

attr(scad,"nexttheta.grouplqa.lmeq1") <- nexttheta.grouplqa.scad.lmeq1
attr(scad,"nexttheta.grouplla.lmeq1") <- nexttheta.grouplla.scad.lmeq1

nexttheta.grouplqa.lasso.lmeq1 <- function(theta,sigma2,y,X,Z,b,n,p,q,penalize,penalty,groupindex,numgroup)  {
  b.mean <- b$mean
  b.var <- b$var
  ni <- sapply(y,nrow)
  out <- .C("nexttheta_grouplqa_lasso_lmeq1",as.double(theta),as.double(sigma2),as.double(unlist(y)),as.double(unlist(X)),as.double(unlist(Z)),
            as.double(unlist(b.mean)),as.double(unlist(b.var)),as.integer(n),as.integer(p),as.integer(q),as.integer(unlist(ni)),
            as.double(penalize),as.double(penalty),as.integer(groupindex-1),as.integer(numgroup),out=double(length=(p+q*(q+1)/2)))$out
  out <- matrix(out,(p+q*(q+1)/2),1)
  out
}
nexttheta.grouplla.lasso.lmeq1 <- function(theta,sigma2,y,X,Z,b,n,p,q,penalize,penalty,groupindex,numgroup)  {
  b.mean <- b$mean
  b.var <- b$var
  ni <- sapply(y,nrow)
  out <- .C("nexttheta_grouplla_lasso_lmeq1",as.double(theta),as.double(sigma2),as.double(unlist(y)),as.double(unlist(X)),as.double(unlist(Z)),
            as.double(unlist(b.mean)),as.double(unlist(b.var)),as.integer(n),as.integer(p),as.integer(q),as.integer(unlist(ni)),
            as.double(penalize),as.double(penalty),as.integer(groupindex-1),as.integer(numgroup),out=double(length=(p+q*(q+1)/2)))$out
  out <- matrix(out,(p+q*(q+1)/2),1)
  out
}

attr(alasso,"nexttheta.grouplqa.lmeq1") <- nexttheta.grouplqa.lasso.lmeq1
attr(alasso,"nexttheta.grouplla.lmeq1") <- nexttheta.grouplla.lasso.lmeq1

lmeq1.lpd <- function(theta,sigma2,y,X,Z,n,p,q,ni,b)  {
  k <- lapply(ni,function(x) rep(1,x))
  out <- .C("lmmeq1_lpd",as.double(theta),as.double(sigma2),as.double(unlist(y)),as.double(unlist(X)),as.double(unlist(Z)),
            as.double(unlist(k)),as.integer(n),as.integer(p),as.integer(q),as.integer(unlist(ni)),as.double(unlist(b)),out=double(length=1))$out
  out
}

lme.sim.theta <- function(theta,sigma2,y,X,Z,b,n,p,q,penalty)  {
  beta <- theta[1:p,1,drop=FALSE]
  m <- theta[(p+1):(p+q*(q+1)/2),1,drop=FALSE]
  M <- ltmat(m)
  Jq <- dupltmat(q)
  fun1 <- function(i)  {
    out <- crossprod(cbind(X[[i]],kronecker(t(b[[i]]),Z[[i]])%*%Jq))
    out
  }
  fun2 <- function(i)  {
    out <- t(cbind(X[[i]],kronecker(t(b[[i]]),Z[[i]])%*%Jq))%*%y[[i]]
    out
  }
  mat1 <- matrix(apply(sapply(1:n,fun1),1,sum),p+q*(q+1)/2,p+q*(q+1)/2)/sigma2
  mat2 <- matrix(apply(sapply(1:n,fun2),1,sum),p+q*(q+1)/2,1)/sigma2
  rg <- rgamma(p+q*(q+1)/2,shape=1,rate=sum(ni)^2*penalty^2)
  D <- solve(mat1+diag(1/rg))
  d <- mat2
  out <- mvrnorm(1,D%*%d,D)
  out
}


ecmeml.lmm  <- function (y, subj, pred, xcol, zcol, vmax, occ, start, maxits = 1000, eps = 1e-04) 
{
    tmp <- table(subj)
    m <- length(tmp)
    nmax <- max(tmp)
    ntot <- length(y)
    pcol <- ncol(pred)
    q <- length(zcol)
    p <- length(xcol)
    {
        if (missing(vmax)) {
            vmax <- diag(rep(1, nmax))
            occ <- integer(ntot)
            iflag <- as.integer(1)
        }
        else iflag <- as.integer(0)
    }
    storage.mode(vmax) <- "double"
    {
        if (!missing(start)) {
            beta <- start$beta
            sigma2 <- start$sigma2
            xi <- start$psi/start$sigma2
            storage.mode(beta) <- "double"
            storage.mode(xi) <- "double"
            storage.mode(sigma2) <- "double"
            sflag <- as.integer(1)
        }
        else {
            beta <- numeric(p)
            xi <- matrix(0, q, q)
            sigma2 <- 0
            sflag <- as.integer(0)
        }
    }
    ##cat("Performing ECME...")
    now <- proc.time()
    tmp <- .Fortran("ecmeml", ntot, as.integer(subj), m, ist = integer(m), 
        ifin = integer(m), as.integer(occ), nmax, vmax, w = array(0, 
            c(nmax, nmax, m)), vinv = array(0, c(nmax, nmax, 
            m)), pcol, as.double(pred), q, as.integer(zcol), 
        ztvinv = array(0, c(q, nmax, m)), ztvinvz = array(0, 
            c(q, q, m)), iflag = iflag, msg = integer(1), u = array(0, 
            c(q, q, m)), iter = integer(1), sflag, sigma2 = sigma2, 
        p, as.integer(xcol), beta = beta, as.double(y), delta = rep(0, 
            ntot), xtw = matrix(0, p, nmax), xtwx = matrix(0, 
            p, p), xtwy = numeric(p), xtwxinv = matrix(0, p, 
            p), wkqq1 = matrix(0, q, q), wkqq2 = matrix(0, q, 
            q), xi = xi, wkqnm = array(0, c(q, nmax, m)), b = matrix(0, 
            q, m), cvgd = integer(1), obeta = rep(0, p), oxi = matrix(0, 
            q, q), maxits = as.integer(maxits), llvec = numeric(as.integer(maxits)), 
        eps = as.double(eps), PACKAGE = "lmm")
    clock <- proc.time() - now
    ##cat("\n")
    iter <- tmp$iter
    msg <- tmp$msg
    {
        if (msg == 1) 
            warning("Supplied V <- i matrix is not positive definite")
        else if (msg == 2) 
            warning("GLS failed for start vals, t(X)%*%inv(V)%*%X not full rank")
        else if (msg == 3) 
            warning("Inadequate information to obtain starting value of psi")
        else if (msg == 4) 
            warning("Value of psi became non-pos.def. during iterations")
        else if (msg == 5) 
            warning("t(X)%*%W%*%X became non-pos.def. during iterations")
    }
    llvec <- tmp$llvec[1:iter]
    converged <- tmp$cvgd == as.integer(1)
    cov.beta <- tmp$xtwxinv * tmp$sigma2
    b.hat <- tmp$b
    cov.b <- tmp$u * tmp$sigma2
    psi <- tmp$xi * tmp$sigma2
    beta <- tmp$beta
    if (!is.null(dimnames(pred)[[2]])) {
        colnames <- dimnames(pred)[[2]]
        names(beta) <- colnames[xcol]
        dimnames(psi) <- list(colnames[zcol], colnames[zcol])
    }
    list(beta = beta, sigma2 = tmp$sigma2, psi = psi, converged = converged, 
        iter = iter, loglik = llvec, cov.beta = cov.beta, b.hat = b.hat, 
        cov.b = cov.b)
}




