
### 02/01/2008 Penalty functions for use in penalized log likelihood estimation


### Hard penalty

hard.penaltyfunction <- function(beta,penalty)  {
  out <- penalty^2 - (beta - penalty)^2*(beta < penalty)
  out
}

hard.gradient <- function(beta,penalty)  {
  out <- -2*(beta - penalty)*(beta < penalty)
  out
}

hard  <- structure(hard.penaltyfunction,gradient=hard.gradient)

### Lasso penalty

lasso.penaltyfunction <- function(beta,penalty)  {
  out <- penalty*beta
  out
}

lasso.gradient <- function(beta,penalty)  {
  beta[] <- penalty
  beta
}

lasso.hessian <- function(beta,penalty)  {
  beta[] <- 0
  beta
}

lasso.density <- function(beta,n,penalty,keepnc=TRUE,log=TRUE)  {
  out <- -penalty*n*abs(beta)
  if (!keepnc) {
    if (log)  return(out) else return(exp(out))
  }
  nc <- log(n*penalty) - log(2)
  out <- out + nc
  if (log) return(out) else return(exp(out))
  out
}

alasso <- structure(lasso.penaltyfunction,gradient=lasso.gradient,hessian=lasso.hessian,density=lasso.density,name="alasso")
  
### Scad penalty

scad.penaltyfunction <- function(beta,penalty)  {
  a <- 3.7
  if (penalty==0)  {
    beta[] <- 0
    return(beta)
  }
  fn <- function(beta)  {
    if (beta <= penalty)   {
      return(penalty*beta)
    }
    else if (beta <= a*penalty)  {
      return(-(beta^2 - 2*a*penalty*beta+penalty^2)/(2*(a-1)))
    }
    else return((a+1)*penalty^2/2)
  }
  out <- sapply(abs(beta),fn)
  out
}

scad.gradient <- function(beta,penalty)  {
  a <- 3.7
  if (penalty==0)  {
    beta[] <- 0
    return(beta)
  }
  fn <- function(beta)  {
    if (beta <= penalty)   {
      return(penalty)
    }
    else if (beta <= a*penalty)  {
      return((a*penalty-beta)/(a-1))
    }
    else return(0)
  }  
  out <- sapply(abs(beta),fn)
  out
}

scad.hessian <- function(beta,penalty)  {
  a <- 3.7
  if (penalty==0)  {
    beta[] <- 0
    return(beta)
  }
  fn <- function(beta)  {
    if (beta <= penalty)   {
      return(0)
    }
    else if (beta <= a*penalty)  {
      return(1/(a-1))
    }
    else return(0)
  }  
  out <- sapply(abs(beta),fn)
  out
}

scad.density <- function(beta,penalty,n,keepnc=TRUE,log=TRUE)  {
  a <- 3.7
  betastar <- 10
  if (penalty == 0) {
    out <- rep(1e-25,length(beta))
    if (log) return(log(out)) else return(out)
  }
  fn <- function(beta)  {
    if (beta <= penalty)   {
      return(-n*penalty*beta)
    }
    else if (beta <= a*penalty)  {
      return(n*(beta^2-2*a*penalty*beta+penalty^2)/(2*(a-1)))
    }
    else if (beta <= betastar)  {
      return(-n*(a+1)*penalty^2/2)
    }
    else return(log(1e-25))
  }
  out <- sapply(abs(beta),fn)
  if (!keepnc) {
    if (log) return(pmax(out,log(rep(1e-25,length(out))))) else return(exp(out))
  }
  nc <- 2*exp(-n*(a+1)*penalty^2/2)*(betastar-a*penalty)+2*(1-exp(-n*penalty^2))/(n*penalty)
  fun <- function(beta) exp(n*(beta^2-2*a*penalty*abs(beta)+penalty^2)/(2*(a-1)))
  nc <- nc + 2*integrate(fun,penalty,a*penalty)$value
  out <- out-log(nc)
  if (log) return(pmax(out,log(rep(1e-25,length(out))))) else return(exp(out))
  out
}

scad <- structure(scad.penaltyfunction,gradient=scad.gradient,hessian=scad.hessian,density=scad.density,name="scad")
### Ridge penalty

ridge.penaltyfunction <- function(beta,penalty)  {
  out <- penalty*beta^2
  out
}
ridge.gradient <- function(beta,penalty)  {
  out <- 2*penalty*beta
  out
}

ridge <- structure(ridge.penaltyfunction,gradient=ridge.gradient)

#################################################################################################################

## penaltyreest estimates random effect penalty estimator
## uses optim function and density functions above

penaltyreest <- function(penalty,penaltyfun,file.xmiss,n,p,nsim,penalize,scalebeta)  {
  file.beta <- sub("xmiss","beta",file.xmiss)
  betas <- read.table(file=file.beta)
  betas <- sweep(betas,2,FUN="*",scalebeta)
  betas  <- betas[,penalize]
  if (attr(penaltyfun,"name")=="alasso")  {
    out <- apply(abs(betas),2,mean)
    out <- mean(out)
    out <- 1/out/n
    return(out)
  }
  fun <- function(penalty)  {
    out <- apply(betas,2,attr(penaltyfun,"density"),n=n,penalty=penalty,keepnc=TRUE)
    out <- sum(apply(out,2,mean))
    out
  }
  mult <- 3
  lam <- exp(seq(log(penalty/mult),log(penalty*mult),length=5))
  if (FALSE)  {
    lam2 <- exp(seq(log(penalty/mult),log(penalty*mult),length=200))
    plot(lam2,sapply(lam2,fun))
    abline(v=min(abs(betas)))
  }
  lamlb <- min(abs(betas))
  fn <- function(start) optim(start,fun,method="L-BFGS-B",lower=lamlb,upper=max(lam),
                              control=list(fnscale=-1))$par
  maxima <- try(sapply(lam,fn))
  if (inherits(maxima, "try-error")) {
    fname <- paste("refailure",sample(1:100,1),".rdata",sep="")
    save(betas,fun,penalty,penaltyfun,nsim,penalize,scalebeta,fn,lam,penalty,file=fname)
    stop(paste("RE est failure see", fname))
  }
  out <- maxima[which.max(sapply(maxima,fun))]
  out
}

