
library(MASS)
source("missing.r")
source("penaltyfunctions.r")

lasso <- function(y,X,penalty)  {
  n <- nrow(X)
  p <- ncol(X)
  beta <- ginv(crossprod(X))%*%t(X)%*%y
  out <- .C("lassoc",as.double(beta),as.double(y),as.double(X),as.double(penalty),as.integer(n),as.integer(p),out=double(length=p))$out
  out
}

reldifffun <- function(a,b)  {
  if (all(a==0))  {
    reldiff <- sqrt(sum(b^2))
  }  else {
    reldiff <- sqrt(sum((a-b)^2)/sqrt(sum((a)^2)))
  }
  reldiff
}

absdifffun <- function(a,b)  {
  if (all(a==0))  {
    reldiff <- sqrt(sum(b^2))
  }  else {
    reldiff <- sqrt(sum((a-b)^2)/sum((a)^2))
  }
  reldiff
}

### iterate takes function at computes beta(k+1)=fun(beta(k)) until some sort convergence

### Arguments--------------------------------------------------------------------------------------------------------
### start, starting value 
### fun, function which computes next iterate
### control$relativeerror, relative error to determine convergence
### control$absoluteerror, absolute error to determine convergence
### control$maxiter, maximum number of iterations
### control$evalfun, function which evaluates convergence, can be null if not used

### Body-----------------------------------------------------------------------------------------------------------
### function keeps computing beta(k+1)=fun(beta(k))
### until either
### 1) the absolute error of the euclidean distance between beta(k+1) and beta(k) is less than control$absoluteerror
### 2) the relative error of the euclidean distance between beta(k+1) and beta(k) is less than control$relativerror
### 3) the relative error of the euclidean distance between control$evalfun(beta(k+1)) and control$evalfun(beta(k))
### is less that control$relativeerror
### 4) k > control$maxiter
### computes does 1) if porvided control params else 2) else 3) else 4)
### control$which.convergence determines which of p variables to decide convergence on
### all nuisance parameters should be passed to argumements in a named fashion. 

iterate <- function(start,fun,control)  {
  ## if fun fails we save everything to debug
  if (is.null(control$iffailurethensave)) control$iffailurethensave <- TRUE
  i <- 1
  theta.cur <- start
  est <- list()
  est[[1]] <- start
  evalfun <- !is.null(control$evalfun)
  if (!is.null(control$absoluteerror))  {
    reldiffun <- absdifffun
    control$relativeerror <- control$absoluteerror
  }  
  repeat  {
    theta.old <- theta.cur
    out <- try(fun(theta.cur))
    ##if fun bombs lets spit out stuff so we can debug
    if (inherits(out, "try-error") | any(is.na(out)) ) {
      if (control$iffailurethensave)  {
        crashfilename <- paste("iritateerror",sample(1:1e3,1),".rdata",sep="")
        cat(paste("There was an error in iterate see file",crashfilename,date()),"\n")
        save(theta.cur,fun,est,file=crashfilename)
      }
      stop("error while performing maximization")
    } else {
      theta.cur <- out
      est[[i+1]] <- out
    }
    i <- i + 1
    if (i <= 2) next
    relerror.par <- reldifffun(est[[i-1]],est[[i]])
    ##suppose I not only want to copmare succesive values but compare a compare values with respect to some
    ##function, evalfun, like the function I am trying to maximize
    if (evalfun) {
      relerror.fun <- reldifffun(control$evalfun(est[[i-1]]),control$evalfun(est[[i]]))
    } else relerror.fun <- control$relativeerror
    if (relerror.par < control$relativeerror)  break
    if (relerror.fun < control$relativeerror)  break
    if (i >= control$maxiter) break
  }
  output <- list(estimate=theta.cur,iterations=est)
  output
}

### simultcondest conditionally computes multiple function until some sort of convergence

### Arguments--------------------------------------------------------------------------------------------------------
### startlist, named list of length p of starting value 
### funlist, names list of length p of functions 
### control$relativeerror, relative error to determine convergence
### control$absoluteerror, absolute error to determine convergence
### control$maxiter, maximum number of iterations
### control$evalfun, function which evaluates convergence, can be null if not used

### Body-----------------------------------------------------------------------------------------------------------
### function keeps computing funlist[[k]](startlist[[k]])
### until either
### 1) the absolute error of the euclidean distance between beta(k+1) and beta(k) is less than control$absoluteerror
### 2) the relative error of the euclidean distance between beta(k+1) and beta(k) is less than control$relativerror
### 3) the relative error of the euclidean distance between control$evalfun(beta(k+1)) and control$evalfun(beta(k))
### is less that control$relativeerror
### 4) k > control$maxiter
### computes does 1) if porvided control params else 2) else 3) else 4)
### control$which.convergence determines which of p variables to decide convergence on
### all nuisance parameters should be passed to argumements in a named fashion. 

### Output---------------------------------------------------------------------------------------------------------
### Spits out estimatevalue it converged to and iterates

                                        
simultcondest <- function(startlist,funlist,arguments,control)  {
  if (is.null(control$print)) {
    control$print <- rep(FALSE,length(startlist))
  } else {
    if (length(control$print)==1)  control$print <- rep(control$print,length(startlist))
  }
  ##determine whether we want to compare relative or absolute difference
  if (!is.null(control$absoluteerror))  {
    reldiffun <- absdifffun
    control$relativeerror <- control$absoluteerror
  }
  p <- length(startlist)
  parameters <-list()
  parameters[[1]] <- currentparameters <- startlist
  i <- 2
  relerror <- matrix(0,1,ncol=p)
  relerrortmp <- vector(length=p)
  names(relerrortmp) <- colnames(relerror) <- names(startlist)
  repeat {
    oldparameters <- currentparameters
    for ( j in 1:p )  {
      tmparguments <- c(currentparameters,arguments,list(control=control))
      tmpfun <- funlist[[j]]
      args <- list()
      namesargs <- names(formals(funlist[[j]]))
      length(args) <- length(namesargs)
      names(args) <- namesargs
      whereldots <- match("...", namesargs)
      namesargs <- namesargs[namesargs != "..."]
      args[namesargs[-1]] <- tmparguments[match(namesargs[-1],names(tmparguments))]
      if (!is.na(whereldots)) {
        args[[whereldots]] <- tmparguments
        names(args)[whereldots] <- ""
      }
      fun <- function(x) {
        if (!is.null(control$debug))  {
          if (is.null(control$whichdebug))
            debug(tmpfun)
          else
            if (j==control$whichdebug) debug(tmpfun)
        }
        arglist <- args
        if (is.null(x)) {
          arglist <- arglist[-1]
          arglist <- c(list(NULL),arglist)
          names(arglist)[[1]] <- namesargs[1]
        } else arglist[[1]] <- x
        do.call("tmpfun",arglist)
      }
      #browser()
      ##debug(tmpfun)
      if ( control$print[j] == TRUE) {
        cat(paste("Started function of parameter ", names(startlist)[j], "at",date()),"\n")
      }
      #Rprof()
      tmp <- try(fun(currentparameters[[j]]))
      #Rprof(NULL)
      if (inherits(tmp, "try-error")) {
        #browser()
        #debug(tmpfun)
        #fun(currentparameters[[j]])
        crashfilename <- paste("simulterror",sample(1:1e3,1),".rdata",sep="")
        cat(paste("There was an error under simultaneous maximization see file",crashfilename,date()),"\n")
        save(j,tmp,fun,tmpfun,args,currentparameters,parameters,file=crashfilename)
        stop("error while performing maximization")
      }
      if ( control$print[j] == TRUE) {
        if (control$which.convergence[j])
          cat(paste("Ending parameter value",paste(round(c(tmp),3),collapse="-"),"at",date()),sep="\n")
      }
      currentparameters[[j]] <- tmp
    }
    parameters[[i]] <- currentparameters
    if (!is.null(control$saveparameters)) parameters[[i]][!control$saveparameters] <- NA
    if ( i >= 3) {
      for ( j in 1:p ) {
        if (!control$which.convergence[j])  {
          relerrortmp[j] <- 0
          next
        }
        relerrortmp[j] <- reldifffun(parameters[[i]][[j]],parameters[[i-1]][[j]])
      }
    } else relerrortmp[] <- control$relativeerror
    ##rewrote function so only first functional parameter has to be subsequently small
    relerror <- rbind(relerror,relerrortmp)
    if (all(relerrortmp < control$relativeerror)) break
    #browser()
    if (i==100) cat("Exceeded 100 iterations in simultcondest","\n")
    if (i > control$maxiter) {
      cat("Exceeded maximum iterations in simultcondest","\n")
      #browser()
      break
    }
    i <- i+1
  }
  output <- list(estimate=currentparameters,iterations=parameters,relerror=relerror)
  output
}

diagonal <- function(x) {
  m <- length(x)
  out <- matrix(0,m,m)
  for ( i in 1:m ) out[i,i] <- x[i]
  out
}

nextbeta.mle <- function(beta,ll.grad,ll.hess)  {
  beta <- beta - ginv(ll.hess(beta))%*%matrix(ll.grad(beta),ncol=1)
  beta 
}
nextbeta.lqa <- function(beta,penalize,pf.grad,ll.grad,ll.hess)  {
  ##since ginv(-ll.hess) is approx covariance matrix of beta, use it as a guide to determine which betas are small
  eps <- .1*sqrt(diag(ginv(-ll.hess(beta))))
  ##only compute components of Sigma0 matrix if beta is not small or if coefficient is not penalized
  not0 <- (abs(beta) > eps) | !penalize
  ##if all are small then return 0 vector
  if (sum(not0) == 0 ) {
    beta[] <- 0
    return(beta)
  }
  ##components which are unpenalized or statistically significant should have penalty gradient, pg, values of 0 
  pg <- ifelse(penalize,pf.grad(abs(beta)),0)
  Sigma0 <- diagonal(c(pg[not0]/abs(beta[not0])))
  out <- ll.hess(beta)[not0,not0,drop=FALSE] - n*Sigma0
  out <- ginv(out)%*%(ll.grad(beta)[not0,drop=FALSE] - n*Sigma0%*%beta[not0,,drop=FALSE])
  out <- beta[not0,,drop=FALSE] - out
  beta[not0,] <- out
  beta[!not0,] <- 0
  beta
}
nextbeta.lla <- function(beta,penalize,pf.grad,ll.grad,ll.hess)  {
  ##since ginv(-ll.hess) is approx covariance matrix of beta, use it as a guide to determine which betas are small
  eps <- .001*sqrt(diag(ginv(-ll.hess(beta))))
  p <- nrow(beta)
  ##determine which components have penalty gradient values of 0
  ind0 <- ifelse(penalize,pf.grad(abs(beta))==0,TRUE)
  Xstar <- chol(-ll.hess(beta))
  ystar <- (Xstar%*%beta + Xstar%*%ginv(crossprod(Xstar))%*%ll.grad(beta))
  ##quadratic approx of log lik is -1/2crossprod(ystar-Xstar%*%beta)
  ##now we use local linear approx on peanlty function
  ##ind0=TRUE then component is unpenalized, i.e. penalty function is constant
  M <- diagonal(c(ifelse(ind0,rep(1,p),1/pf.grad(abs(beta)))))
  ## if penalty is constant w.r. to beta then that is equiv to no penalty function
  if (sum(ind0)==p) {
    out <- matrix(lm(ystar~Xstar-1)$coef,p,1)
    return(out)
  }
  Xstar <- Xstar%*%M
  XU <- Xstar[,ind0,drop=FALSE]
  XVstar <- Xstar[,!ind0,drop=FALSE]
  if (sum(ind0)==0) HU <- matrix(0,p,p) else HU <- XU%*%ginv(crossprod(XU))%*%t(XU)
  ystar2 <- ystar - HU%*%ystar
  XVstar2 <- XVstar - HU%*%XVstar
  betahatV <- lasso(ystar2,XVstar2,2*n)
  if (sum(ind0)==0) {
    out <- M%*%betahatV
      return(out)
  }
  betahatU <- matrix(lm((ystar-XVstar%*%betahatV)~XU-1)$coef,ncol=1)
  out <- matrix(0,nrow=p)
  out[ind0] <- betahatU
  out[!ind0] <- M[!ind0,!ind0,drop=FALSE]%*%betahatV
  out
}
groupdistance <- function(beta,groupindex,numgrp)  {
  K <- numgrp
  lengrp <- distgrp <- vector(length=K)
  for (i in 1:K)  {
    lengrp[i] <- nrow(beta[groupindex==i,,drop=FALSE])
    distgrp[i] <- sqrt(crossprod(beta[groupindex==i,,drop=FALSE]))
  }
  lengrp <- lengrp[groupindex]
  distgrp <- distgrp[groupindex]
  out <- list(lengrp=lengrp,distgrp=distgrp)
  out
}
  
    
## performs the group lasso
## group is a vector of length beta which inicates to which group each element of beta belongs, if no groups then it should be 1:p
nexttheta.grouplqa <- function(theta,penalty,penalize,groupindex,numgrp,pf.grad,ll.grad,ll.hess,n)  {
  p <- nrow(theta)
  ##since ginv(-ll.hess) is approx covariance matrix of theta, use it as a guide to determine which thetas are small
  outgrp <- groupdistance(theta,groupindex,numgrp)
  eps <- .1*sqrt(diag(ginv(-ll.hess(theta))))
  groupeps <- groupdistance(matrix(eps,,1),groupindex,numgrp)$distgrp
  groupnot0 <- outgrp$distgrp > (.15)^(1/2) | !penalize
  if (all(!groupnot0)) {
    theta[] <- 0
    return(theta)
  }
  lengrp <- outgrp$lengrp
  distgrp <- outgrp$distgrp
  ##not0 <- c(abs(theta)) > eps 
  ##only compute components of Sigma0 matrix if theta_k is not small or if coefficient is not penalized
  ##if all are small then return 0 vector
  ##if (!any(not0) ) {
  ##  theta[] <- 0
  ##  return(theta)
  ##}
  ##components which are unpenalized or statistically significant should have penalty gradient, pg, values of 0
  fun <- function(i) sqrt(lengrp)[i]*pf.grad(sqrt(lengrp)[i]*distgrp[i],penalty=penalty[i])/(distgrp[i]*lengrp[i])
  lqaapprox <- sapply(1:p,fun)
  pg <- ifelse(penalize,lqaapprox,0)
  Sigma0 <- diagonal(c(pg[groupnot0]))
  out <- ll.hess(theta)[groupnot0,groupnot0,drop=FALSE] - n*Sigma0
  out <- ginv(out)%*%(ll.grad(theta)[groupnot0,drop=FALSE] - n*Sigma0%*%theta[groupnot0,,drop=FALSE])
  out <- theta[groupnot0,,drop=FALSE] - out
  theta[groupnot0,] <- out
  theta[!groupnot0,] <- 0
  theta
}

maxpenalty <- 30

recode <- function(x,oldvalues,newvalues)  {
  n <- length(x)
  p <- length(oldvalues)
  x.new <- x.old <- x
  for (i in 1:p)  {
    x.new[x.old==oldvalues[i]] <- newvalues[i]
  }
  x.new
}

nexttheta.grouplla <- function(theta,penalty,penalize,groupindex,numgrp,pf.grad,ll.grad,ll.hess,p,n)  {
  if (any(theta==0))  {
    not0 <- theta!=0
    ll.grad.new <- function(theta.not0) {
      theta <- matrix(0,p,1)
      theta[not0,] <- theta.not0
      out <- ll.grad(theta)[not0,,drop=FALSE]
      out 
    }
    ll.hess.new <- function(theta.not0) {
      theta <- matrix(0,p,1)
      theta[not0,] <- theta.not0
      out <- ll.hess(theta)[not0,not0,drop=FALSE]
      out
    }
    numgroup.new <- length(unique(groupindex[not0]))
    groupindex.new <- recode(groupindex[not0],unique(groupindex[not0]),1:numgroup.new)
    out <- nexttheta.grouplla(theta[not0,,drop=FALSE],penalty[not0],penalize[not0],groupindex.new,numgroup.new,pf.grad,
                                            ll.grad.new,ll.hess.new,sum(not),n)
    theta <- matrix(0,p,1)
    theta[not0,] <- out
    return(theta)
  }
  p <- nrow(theta)
  outgrp <- groupdistance(theta,groupindex,numgrp)
  lengrp <- outgrp$lengrp
  distgrp <- outgrp$distgrp
  if (all(distgrp==0)) {
    theta[] <- 0
    return(theta)
  }
  ##determine which components have penalty gradient values of 0
  Xstar <- chol(-ll.hess(theta))
  ystar <- (Xstar%*%theta + Xstar%*%ginv(crossprod(Xstar))%*%ll.grad(theta))
  ##quadratic approx of log lik is -1/2crossprod(ystar-Xstar%*%theta)
  ##now we use local linear approx on peanlty function
  ##ind0=TRUE then component is unpenalized, i.e. penalty function is constant
  fun <- function(i) sqrt(lengrp)[i]*pf.grad(sqrt(lengrp)[i]*distgrp[i],penalty=penalty[i])*abs(c(theta)[i])/distgrp[i]
  llaapprox <- sapply(1:p,fun)
  ## lla approximation of group which is already zero, we supply large penalty values
  llaapprox <- ifelse(distgrp==0,maxpenalty,llaapprox)
  pfg0 <- ifelse(penalize,llaapprox==0,TRUE)
  M <- diagonal(c(ifelse(pfg0,rep(1,p),1/llaapprox)))
  ## if penalty is constant w.r. to theta then that is equiv to no penalty function
  if (sum(pfg0)==p) {
    out <- matrix(lm(ystar~Xstar-1)$coef,p,1)
    return(out)
  }
  Xstar <- Xstar%*%M
  XU <- Xstar[,pfg0,drop=FALSE]
  XVstar <- Xstar[,!pfg0,drop=FALSE]
  if (sum(pfg0)==0) HU <- matrix(0,p,p) else HU <- XU%*%ginv(crossprod(XU))%*%t(XU)
  ystar2 <- ystar - HU%*%ystar
  XVstar2 <- XVstar - HU%*%XVstar
  thetahatV <- lasso(ystar2,XVstar2,2*n)
  if (sum(pfg0)==0) {
    out <- M%*%thetahatV
      return(out)
  }
  thetahatU <- matrix(lm((ystar-XVstar%*%thetahatV)~XU-1)$coef,ncol=1)
  out <- matrix(0,nrow=p)
  out[pfg0] <- thetahatU
  out[!pfg0] <- M[!pfg0,!pfg0,drop=FALSE]%*%thetahatV
  out
}



### penloglikest performs penalized likelihood estimation, Fan and Li (2001), using the local linear approximation of
### Zou and Li (2007)

### Arguments--------------------------------------------------------------------------------------------------------
### beta, is starting value which can be NULL
### loglik, is loglikelihood function with attributes "gradient", "hessian"
### penaltyfunction is function with attribtues "gradient" and "density" see penaltyfunctions.r
### penalty.par is value of penalty parameter
### control is list of control values to determine convergence which are passed to function iterate
### ldots are alternative arguments which may need to be passed to function
### ldots$penalize, vector of length(beta) which indicates which regression coefficients are to be penalized

### Body-----------------------------------------------------------------------------------------------------------
### function uses obth local quadratic approximation and local linear approximations 
### since both of these approximations are VERY sensitive to starting values I use multiple starting values.
### One of these starting values is the penalized log liklihood estimate resulting from using the local quadratic
### approximation of Li and Fan (2001). 

### Output---------------------------------------------------------------------------------------------------------
### Spits out a maxima with PML estimate and iterates

grouppenloglikest <- function(theta=NULL,penalty.par,loglik,penalty.fun,groupindex,control,...)  {
  if (is.list(c(...))) ldots <- c(...) else ldots <- list(...)
  p <- nrow(theta)
  ll <- loglik
  ll.grad <- attr(loglik,"gradient")
  ll.hess <- attr(loglik,"hessian")
  pf.fun <- penalty.fun
  pf.grad <- attr(penalty.fun,"gradient")
  ##lets extract n sample size and which coefficients we are penalizing
  n <- ldots[["n"]]
  ##if penalty paramter is zero  
  if (is.null(ldots[["penalize"]])) penalize <- rep(TRUE,p) else penalize <- ldots[["penalize"]]
  formals(ll)[-1] <- ldots[names(formals(ll))[-1]]
  formals(ll.grad)[-1] <- ldots[names(formals(ll.grad))[-1]]
  formals(ll.hess)[-1] <- ldots[names(formals(ll.hess))[-1]]
  formals(pf.fun)[-1] <- ldots[names(formals(pf.fun))[-1]]
  formals(pf.grad)[-1] <- ldots[names(formals(pf.grad))[-1]]
  ##lets create penalized loglikelihood function
  ###  mlefun computes unpenalized mle 
  ##nexttheta.lla computes local linear approximation, Zou and Li (2007) Annals
  #control$mle <- control
  #control$maxiter <- 5
  ls.est <- theta
  mlefun <- function(theta) nextbeta.mle(theta,ll.grad,ll.hess)
  mle.est <- iterate(ls.est,mlefun,control)$estimate
  ##if penalty == 0 return unpenalized
  if (all(penalty.par==0)) {
    maxima <- cbind(ls.est,mle.est)
    funval <- sapply(1:ncol(maxima),function(i) ll(maxima[,i,drop=FALSE]))
    out <- maxima[,which.max(funval),drop=FALSE]
    return(out)
  }
  ##debug(nexttheta.lqagroup)
  numgrp <- length(unique(groupindex))
  lqafun <- function(theta) {
    out <- try(nexttheta.grouplqa(theta,penalty.par,penalize,groupindex,numgrp,pf.grad,ll.grad,ll.hess))
    if (inherits(out,"try-error"))
        browser()
    out
  }    
  lqa.est <- iterate(mle.est,lqafun,control)$estimate
  startval <- cbind(ls.est,mle.est,lqa.est)
  ##lets creat different starting values off of lqa/starting value
  ##for (i in 1:nrow(mle.est)) {
  ## tmp <- matrix(0,ncol(X),1)  
  ## tmp[i,] <- lqa.est[i,]
  ## startval <- cbind(startval,tmp)
  #}
  ##lets employ lla at these different starting values
  ##debug(nexttheta.grouplla)
  llafun <- function(theta) {
    out <- try(nexttheta.grouplla(theta,penalty.par,penalize,groupindex,numgrp,pf.grad,ll.grad,ll.hess))
    if (inherits(out,"try-error"))
        browser()
    out
  }    
  fun <- function(i) {
    out <- try(iterate(startval[,i,drop=FALSE],llafun,control)$estimate)
    if (inherits(out,"try-error"))
        browser()
    out
  }
  maxima <- sapply(1:ncol(startval),fun)
  ##lets also include ls, mle, and lqa estimates as possible maxima
  maxima <- cbind(maxima,startval)
  ##note I always use the original function not approximations to determine which is maximum value
  pl <- function(theta) {
    gd <- groupdistance(theta,groupindex,numgrp)
    out <- ll(theta) - n*sum(sapply(1:p,function(i) pf.fun(sqrt(gd$lengrp[i])*gd$distgrp[i],penalty=penalty.par[i])))
    out
  }
  funval <- sapply(1:ncol(maxima),function(i) pl(maxima[,i,drop=FALSE]))
  out <- maxima[,which.max(funval),drop=FALSE]
  out
}

penloglikest <- function(beta=NULL,penalty.par,loglik,penalty.fun,control,...)  {
  if (is.list(c(...))) ldots <- c(...) else ldots <- list(...)
  p <- nrow(beta)
  ll <- loglik
  ll.grad <- attr(loglik,"gradient")
  ll.hess <- attr(loglik,"hessian")
  pf.fun <- penalty.fun
  pf.grad <- attr(penalty.fun,"gradient")
  ldots <- c(list(penalty=penalty.par),ldots)
  ##lets extract n sample size and which coefficients we are penalizing
  n <- ldots[["n"]]
  ##if penalty paramter is zero  
  if (is.null(ldots[["penalize"]])) penalize <- rep(TRUE,p) else penalize <- ldots[["penalize"]]
  formals(ll)[-1] <- ldots[names(formals(ll))[-1]]
  formals(ll.grad)[-1] <- ldots[names(formals(ll.grad))[-1]]
  formals(ll.hess)[-1] <- ldots[names(formals(ll.hess))[-1]]
  formals(pf.fun)[-1] <- ldots[names(formals(pf.fun))[-1]]
  formals(pf.grad)[-1] <- ldots[names(formals(pf.grad))[-1]]
  ##lets create penalized loglikelihood function
  pl <- function(x) ll(x) - n*sum(pf.fun(abs(x)))
  ###  mlefun computes unpenalized mle 
  ##nextbeta.lla computes local linear approximation, Zou and Li (2007) Annals
  #control$mle <- control
  #control$maxiter <- 5
  ls.est <- beta
  mlefun <- function(beta) nextbeta.mle(beta,ll.grad,ll.hess)
  mle.est <- iterate(ls.est,mlefun,control)$estimate
  ##if penalty == 0 return unpenalized
  if (penalty.par==0) {
    maxima <- cbind(ls.est,mle.est)
    funval <- sapply(1:ncol(maxima),function(i) ll(maxima[,i,drop=FALSE]))
    out <- maxima[,which.max(funval),drop=FALSE]
    return(out)
  }
  lqafun <- function(beta) nexttheta.grouplqa(beta,penalty,penalize,groupindex,numgrp,pf.grad,ll.grad,ll.hess)
  lqa.est <- iterate(mle.est,lqafun,control)$estimate
  startval <- cbind(ls.est,mle.est,lqa.est)
  ##lets creat different starting values off of lqa/starting value
  ##for (i in 1:nrow(mle.est)) {
  ## tmp <- matrix(0,ncol(X),1)  
  ## tmp[i,] <- lqa.est[i,]
  ## startval <- cbind(startval,tmp)
  #}
  startval <- cbind(startval,matrix(0,p,1))
  ##lets employ lla at these different starting values
  llafun <- function(beta) nextbeta.lla(beta,penalize,pf.grad,ll.grad,ll.hess)
  maxima <- sapply(1:ncol(startval),function(i) iterate(startval[,i,drop=FALSE],llafun,control)$estimate)
  ##lets also include ls, mle, and lqa estimates as possible maxima
  maxima <- cbind(maxima,startval)
  ##note I always use the original function not approximations to determine which is maximum value
  funval <- sapply(1:ncol(maxima),function(i) pl(maxima[,i,drop=FALSE]))
  out <- maxima[,which.max(funval),drop=FALSE]
  out
}


########################################################################################################################

bicpenalty <- function(beta,n)  {
  df <- sum(beta!=0) 
  out <- list(aic=2*df,bic=df*log(n))
  out <- out$bic
  out
}

## icqpenlatyest crudely estimates penalty parameter using icq criterion
## fun is function which computes icq for given penalty parameter
## niter - length of grid value of penalty
## penlaty.start - starting value
## multy - multiplier of starting value
## control
## 

icqpenaltyest <- function(fun,penalty.start,niter,multy,cuts=4,control)  {
  if (is.null(control$plot)) control$plot <- FALSE
  penaltylim <- c(lower=penalty.start/multy,upper=penalty.start*multy)
  i <- 0
  cuts.orig <- cuts
  if (!is.null(control$absoluteerror))  {
    reldiffun <- absdifffun
    control$relativeerror <- control$absoluteerror
  }
  repeat  {
    i <- i + 1
    lam <- exp(seq(log(penaltylim["lower"]),log(penaltylim["upper"]),length=niter))
    value <- sapply(lam,fun)
    ## ok why don't we use previous iterations lams and values
##     if (i>1)  {
##       browser()
##       ind <- min(lam) < lam.prev & lam.prev < max(lam)
##       lam.prev <- lam.prev[ind]
##       value.prev <- value.prev[ind]
##       lam <- c(lam,lam.prev)
##       value <- c(value,value.prev)
##       ind <- order(lam)
##       lam <- lam[ind]
##       value <- value[ind]
##     }
    relerror.penalty <- max(sapply(2:niter,function(i) reldifffun(lam[i],lam[i-1])))
    relerror.val <- reldifffun(max(value),min(value))
    ## plots search of lambda
    ## control$plot <- TRUE
    if (control$plot)  {
      plot(lam,value)
      points(lam[order(value)[1:ceiling(niter/4)]],value[order(value)[1:ceiling(niter/4)]],col="blue")
      points(lam[order(value)[1:ceiling(niter/10)]],value[order(value)[1:ceiling(niter/10)]],col="red")
      #browser()
    }
    if ( relerror.penalty < control$relativeerror) break
    if ( relerror.val < control$relativeerror) break
    if ( i >= 25 )  {
      #save(y,X,penalty,penaltyfun,modelpenalty,control,scalebeta,ldots,file="erroricq.rdata")
      cat(paste("error in geeting icq estimate"),"\n")
      break
    }
    cuts <- cuts.orig
    repeat  {
      lamcuts <- lam[order(value)][1:ceiling(niter/cuts)]
      if (!all(penaltylim[c("lower","upper")] == range(lamcuts) ))  {
        penaltylim[c("lower","upper")] <- range(lamcuts)
        break
      } else cuts <- cuts + 1
    }
    lam.prev <- lam
    value.prev <- value
    #browser()
  }
  out <- list(estimate=list(penalty=lam[which.min(value)]))
  out
}

icq2penaltyest <- function(fun,penalty.start,niter,multy,cuts=4,control)  {
  if (is.null(control$plot)) control$plot <- FALSE
  penalty1lim <- c(lower=penalty.start[1]/multy,upper=penalty.start[1]*multy)
  penalty2lim <- c(lower=penalty.start[2]/multy,upper=penalty.start[2]*multy)  
  i <- 0
  cuts.orig <- cuts
  if (!is.null(control$absoluteerror))  {
    reldiffun <- absdifffun
    control$relativeerror <- control$absoluteerror
  }
  repeat  {
    i <- i + 1
    lam1 <- c(exp(seq(log(penalty1lim["lower"]),log(penalty1lim["upper"]),length=ifelse(penalty1lim["lower"]==penalty1lim["upper"],1,niter))),
	penalty.start[1])
    lam2 <- c(exp(seq(log(penalty2lim["lower"]),log(penalty2lim["upper"]),length=ifelse(penalty2lim["lower"]==penalty2lim["upper"],1,niter))),
	penalty.start[2])
    ##lam1 <- seq(penalty1lim["lower"],penalty1lim["upper"],length=ifelse(penalty1lim["lower"]==penalty1lim["upper"],1,niter))
    ##lam2 <- seq(penalty2lim["lower"],penalty2lim["upper"],length=ifelse(penalty2lim["lower"]==penalty2lim["upper"],1,niter))
    lam <- expand.grid(lam1=lam1,lam2=lam2)
    value <- sapply(1:nrow(lam),function(i) fun(unlist(lam[i,])))
    ## ok why don't we use previous iterations lams and values
##     if (i>1)  {
##       browser()
##       ind <- min(lam) < lam.prev & lam.prev < max(lam)
##       lam.prev <- lam.prev[ind]
##       value.prev <- value.prev[ind]
##       lam <- c(lam,lam.prev)
##       value <- c(value,value.prev)
##       ind <- order(lam)
##       lam <- lam[ind]
##       value <- value[ind]
##     }
    if (!(penalty1lim["lower"]==penalty1lim["upper"]))  {
      relerror.penalty1 <- max(sapply(2:niter,function(i) reldifffun(lam1[i],lam1[i-1])))
    }  else relerror.penalty1 <- 0
    if (!(penalty2lim["lower"]==penalty2lim["upper"]))  {
      relerror.penalty2 <- max(sapply(2:niter,function(i) reldifffun(lam2[i],lam2[i-1])))
    }  else relerror.penalty2 <-  0   
    relerror.val <- reldifffun(max(value),min(value))
    ## plots search of lambda
    ## control$plot <- TRUE
    if (control$plot)  {
      plot(lam,value)
      points(lam[order(value)[1:ceiling(niter/4)]],value[order(value)[1:ceiling(niter/4)]],col="blue")
      points(lam[order(value)[1:ceiling(niter/10)]],value[order(value)[1:ceiling(niter/10)]],col="red")
      #browser()
    }
    if ( relerror.penalty1  < control$relativeerror & relerror.penalty2  < control$relativeerror ) break
    if ( relerror.val < control$relativeerror/5) break
    if ( i >= 25 )  {
      #save(y,X,penalty,penaltyfun,modelpenalty,control,scalebeta,ldots,file="erroricq.rdata")
      cat(paste("error in geeting icq estimate"),"\n")
      break
    }
    cuts <- cuts.orig
    repeat  {
      lam1cuts <- lam[order(value)[1:ceiling(nrow(lam)/cuts)],1]
      if (!all(penalty1lim[c("lower","upper")] == range(lam1cuts) ) | range(lam1cuts)[1]==range(lam1cuts)[2])  {
        if (relerror.penalty1 < control$relativeerror) lamcuts1 <- lam[which.min(value),1] 
        penalty1lim[c("lower","upper")] <- range(lam1cuts)
        break
      } else cuts <- cuts + 1
    }
    cuts <- cuts.orig
    repeat  {
      lam2cuts <- lam[order(value)[1:ceiling(nrow(lam)/cuts)],2]
      if (!all(penalty2lim[c("lower","upper")] == range(lam2cuts) ) | range(lam2cuts)[1]==range(lam2cuts)[2])  {
        if (relerror.penalty2 < control$relativeerror) lamcuts2 <- lam[which.min(value),1]
        penalty2lim[c("lower","upper")] <- range(lam2cuts)
        break
      } else cuts <- cuts + 1
    }
    cat("lamcuts1",penalty1lim,",lamcuts2",penalty2lim,date(),"\n")
  }
  out <- list(estimate=list(penalty=c(lam[which.min(value),1],lam[which.min(value),2])))
  out
}




reconstructX <- function(Xorig,Xsim)  {
  if (all(!is.na(Xorig))) return(Xorig)
  longX <- c(t(Xorig))
  longX[is.na(longX)] <- Xsim
  out <- matrix(unlist(longX),nrow(Xorig),ncol(Xorig),byrow=TRUE)
  out
}

